       PROGRAM BOMTOM
C******************************************************************************
C******************************************************************************
C  BOMTOM (BUREAU OF MINES TOMOGRAPHY)IS A SIMULTANEOUS ITERATIVE
C   RECONSTRUCTION TECHNIQUE (SIRT) TOMOGRAPHIC PROGRAM FOR CROSSHOLE
C   SEISMIC TRAVEL TIMES.  BOMTOM WILL GENERATE SYNTHETIC TRAVEL TIMES
C   IF THE ITERATION LIMIT EQUALS -1.
C  INPUT, OUTPUT AND DATA FILE NAMES ARE ENTERED FROM THE KEYBOARD. INTERACTIVE
C   INSTRUCTIONS ARE GIVEN FOR CREATING A NEW INPUT FILE IF THE SPECIFIED INPUT
C   FILE IS NOT FOUND.  THERE IS AN OUTPUT FILE WITH TITLES FOR PRINTING, AND A
C   DATA FILE IN WHICH RESULTS ARE SAVED EVERY 10 ITERATIONS AND AT THE END.
C   THIS DATA FILE CAN BE USED AS THE INPUT FILE WHEN RESUMING A RUN.
C  OPTIONS CAN BE CHANGED FROM THE KEYBOARD INTERACTIVELY.  OPTIONS INCLUDE
C   APPLYING CONSTRAINTS, SMOOTHING, AND CHANGING INITIAL VELOCITY GUESSES.
C  EXECUTION CAN BE STOPPED BY HOLDING CONTROL-BREAK UNTIL BOMTOM WRITES TO
C   THE SCREEN, EVERY 10 ITERATIONS.
C  CORRECTION FACTORS ARE WEIGHTED BY THE FRACTION OF A GIVEN RAY IN A GIVEN
C   PIXEL (CELL) DIVIDED BY THE SUM OF FRACTIONS OF RAY PATHS IN THAT PIXEL,
C   AND BY AN OPTIONAL WEIGHT ACCOUNTING FOR RELATIVE RELIABILITY OF DATA.
C  WRITTEN BY DARYL TWEETON, U S BUREAU OF MINES, MINNEAPOLIS, MN, 1986.
C   LATEST REVISION NOV, 1991. BOMTOM PRODUCES A GRID FOR SURFER CONTOURS.
C
C  THE BUREAU OF MINES EXPRESSLY DECLARES THAT THERE ARE NO WARRANTIES
C   EXPRESSED OR IMPLIED WHICH APPLY TO THE SOFTWARE CONTAINED HEREIN.  BY
C   ACCEPTANCE AND USE OF SAID SOFTWARE, WHICH IS CONVEYED TO THE USER WITHOUT
C   CONSIDERATION BY THE BUREAU OF MINES, THE USER EXPRESSLY WAIVES ANY AND
C   ALL CLAIMS FOR DAMAGE AND/OR SUITS FOR OR BY REASON OF PERSONAL INJURY, OR
C   PROPERTY DAMAGES, INCLUDING SPECIAL, CONSEQUENTIAL, OR OTHER SIMILAR
C   DAMAGES ARISING OUT OF OR IN ANY WAY CONNECTED WITH THE USE OF THE SOFTWARE
C   CONTAINED HEREIN.
C*****************************************************************************
C  OPERATION OF PROGRAM                                   LABEL NUMBERS
C   READ OR CREATE INPUT FILE                                100-299
C   WRITE OPTIONS AND DATA IN OUTPUT FILE                    300-399
C   CALCULATE GENERAL GEOMETRIC FACTORS                      400-499
C   CALCULATE LENGTHS OF PATH SEGMENTS                       500-599
C   ACCUMULATE CORRECTION FACTORS FOR EACH PIXEL             600-649
C   APPLY WEIGHTED CORRECTION FACTORS TO VELOCITIES          650-699
C   AVERAGING AND SMOOTHING OPTIONS                          700-799
C   SAVE RESULTS IN DATA FILE                                800-899
C   WRITE RESULTS IN OUTPUT FILE                             900-999
C******************************************************************************
C  DESCRIPTION OF DATA INPUT READ FROM INPUT FILE.
C
C  UNITS FOR LENGTH, TRAVEL TIMES, AND VELOCITIES MUST BE SELF-CONSISTENT.
C  USUAL UNITS ARE LENGTH IN METERS, TIME IN MILLISECONDS, VELOCITY IN KM/SEC.
C  FOR LABELING IN THE PROGRAM, TRANSMITTERS ARE ASSUMED TO BE ON LEFT AND
C   RECEIVERS ON RIGHT SIDE.
C  PIXEL 1 IS AT TOP LEFT.  PIXEL 2 IS NEXT PIXEL IN TOP ROW.
C
C  ALL DATA ARE IN LIST-DIRECTED FORMAT.  INPUT NUMBERS MUST BE SEPARATED BY
C   A SPACE OR COMMA.  THEY DO NOT NEED TO BE IN PARTICULAR COLUMNS.
C  ***************************************************************************
C  HEADER
C
C  ONE LINE OF COMMENTS FOR THE TOP OF OUTPUT FILE.  LEAVE FIRST COLUMN BLANK.
C*****************************************************************************
C  MDATA     MWT     MSTOR     MCOR    MBORE     MSMTH     MDIF
C
C  USE MDATA=0 TO INPUT TRAVEL TIMES SEPARATELY FROM TRANSMITTER AND RECEIVER
C   POSITIONS WITH POSITION IDENTIFICATION NUMBERS.  IDENT NUMBERS NEED NOT
C   BE SEQUENTIAL, BUT MUST BE INTEGERS WITH ABSOLUTE VALUE LESS THAN 32767.
C  USE MDATA=1 FOR POSITIONS AND CORRESPONDING TIMES ON THE SAME LINE.
C  SET MWT=1 TO USE INDIVIDUAL WEIGHTING FACTORS FOR ACCURACY OF TRAVEL TIMES.
C  USE MSTOR=0 TO RECALCULATE PATH SEGMENTS EACH ITERATION OR =1 TO STORE AND
C   RECALL PATH SEGMENTS (MORE MEMORY BUT FASTER EXECUTION). SEE COMMENTS
C   FOLLOWING DIMENSION STATEMENT.
C  USE MCOR=0 FOR MULTIPLICATIVE CORRECTION FACTORS AND =1 FOR ADDITIVE.
C   THEY GIVE SIMILAR RESULTS.
C  SET MBORE=1 TO FIX VELOCITIES IN PIXELS CONTAINING TRANSMITTER OR RECEIVER
C   POSITIONS TO THE INITIAL VALUES.  WOULD BE USED WITH BOREHOLE SONIC LOGS.
C  SET MSMTH=1 FOR SMOOTHING AFTER EACH ITERATION.  WEIGHTING IS 20, 4, 1
C   FOR PIXEL, ITS NEAREST NEIGHBORS, AND CORNER NEIGHBORS, RESPECTIVELY.
C  SET MDIF=1 TO FIND DIFFERENCE IN VELOCITIES BETWEEN THIS AND A PRIOR RUN.
C  ****************************************************************************
C  ITRMAX    ITRSAV     TOLABS      TOLINC     RELAX
C
C  ITRMAX = MAXIMUM NUMBER OF ITERATIONS ALLOWED.  SET ITRMAX = -1 TO GENERATE
C   SYNTHETIC TRAVEL TIMES FOR THE INITIAL VELOCITIES.
C  ITRSAV = NUMBER OF ITERATIONS THAT HAVE BEEN PERFORMED; 0 WHEN STARTING
C   A NEW RUN, NOT 0 WHEN CONTINUING RUN USING THE DATA FILE AS INPUT.
C  TOLABS AND TOLINC ARE CONVERGENCE CRITERIA FOR ENDING ITERATIONS. ITERATIONS
C   STOP IF THE AVERAGE ABSOLUTE VALUE OF THE DIFFERENCE BETWEEN CALCULATED AND
C   EXPERIMENTAL TRAVEL TIMES IS LESS THAN TOLABS, OR IF THE DECREASE IN THAT
C   AVERAGE ERROR IS LESS THAN TOLINC FOR TWO SUCCESSIVE ITERATIONS.
C  RELAX = RELAXATION PARAMETER.  LEAVING RELAX=1.0 IS SAFE, THOUGH
C   CONVERGENCE MAY BE QUICKER IF IT IS ADJUSTED.
C  ****************************************************************************
C  IPDAT      IPIVG      IPFRC      IPERR      IPSEG     IPGRA     ISGRA
C
C  IPDAT=1 CAUSES PRINTING OF POSITIONS, TRAVEL TIMES, AND WEIGHTS.
C  IPIVG=1 CAUSES PRINTING OF INITIAL VELOCITY GUESSES.
C  IPFRC=1 CAUSES PRINTING OF THE SUM OF THE FRACTIONS OF PATHS IN EACH PIXEL.
C  IPERR=1 CAUSES PRINTING OF THE ERRORS (DELTA) FOR EACH TRAVEL TIME.
C  IPSEG=1 CAUSES PRINTING OF SEGMENT LENGTHS FOR EACH PATH AND PIXEL.
C  IPGRA=1 CAUSES PRINTING OF GRAPHICAL DOT MATRIX DISPLAY OF VELOCITIES.
C  ISGRA=1 CAUSES GRAPHICAL SCREEN DISPLAY OF VELOCITIES EVERY 10 ITERATIONS.
C   ANY VALUE OTHER THAN 1 SUPPRESSES THE CORRESPONDING PRINTING OR DISPLAY.
C  ****************************************************************************
C  POSP1X    POSP1Y    POSP1Z    POSPNX   POSPNY    POSPNZ

C  POSP1X, POSP1Y, AND POSP1Z ARE THE X, Y, AND Z COORDINATES OF THE CENTER
C   OF PIXEL 1 (THE TOP LEFT PIXEL).
C  POSPNX, POSPNY, AND POSPNZ ARE THE X, Y, AND Z COORDINATES OF THE CENTER
C   OF PIXEL NPIXH*NPIXV (THE BOTTOM RIGHT PIXEL).
C  THEY SHOULD BE SET SO THAT THE TRANSMITTERS AND RECEIVERS ARE CENTERED AS
C   WELL AS POSSIBLE IN PIXELS.  IF PIXEL 1 IS TOO FAR ABOVE OR TO THE LEFT OF
C   THE PATHS, THEN PIXEL INDICES MAY BE LARGER THAN THE DIMENSION. IF PIXEL 1
C   IS BELOW OR TO THE RIGHT OF PATHS, THEN PIXEL INDICES WILL BE NEGATIVE.
C   CONVERSE DIRECTIONS APPLY TO PIXEL NPIXH*NPIXV.
C  ***************************************************************************
C  NPIXH      NPIXV      NLITOP     NLIBOT      NFXLFT      NFXRHT
C
C  NPIXH=NUMBER OF COLUMNS OF PIXELS (COUNTING HORIZONTALLY).
C  NPIXV=NUMBER OF ROWS OF PIXELS (COUNTING VERTICALLY).
C   IF DESIRED PIXEL HEIGHT IS EPIXV, SET NPIXV=1+(POSPNZ-POSP1Z)/EPIXV.
C  NLITOP AND NLIBOT ARE THE NUMBER OF LATERALLY INVARIANT ROWS AT TOP AND
C   BOTTOM OF THE GRID.  THE VELOCITY IN EACH OF THOSE ROWS IS SET EQUAL TO
C   THE AVERAGE FOR THAT ROW AFTER EACH ITERATION.
C  NFXLFT AND NFXRHT ARE THE NUMBER OF COLUMNS AT THE LEFT AND RIGHT SIDES OF
C   THE GRID IN WHICH THE VELOCITIES ARE FIXED AT THE INITIAL VALUES.
C  ***************************************************************************

C  THE INPUT OF TRANSMITTER AND RECEIVER LOCATIONS DEPENDS ON MDATA AND ITRMAX.
C   **************************************
C   IF MDATA = 0 OR ITRMAX = -1, USE THE FOLLOWING FORM:
C
C   NPST         NPSR         ADJDIS
C
C   NPST, NPSR ARE THE NUMBER OF TRANSMITTER, RECEIVER POSITIONS
C   ADJDIS IS ADDED TO ALL HORIZONTAL DISTANCES BETWEEN BOREHOLES.  IT IS ADDED
C    TO THE RECEIVER POSITION.
C   ********
C   IDPST(IPST)   POSTX(IPST)   POSTY(IPST)   POSTZ(IPST)
C    (ONE LINE FOR EACH IPST = 1 TO NPST)
C   IDPST(IPST) IS THE IDENTIFICATION NUMBER OF THE TRANSMITTER POSITION.
C    THEY DO NOT NEED TO BE SEQUENTIAL.
C   POSTX, POSTY, AND POSTZ ARE X, Y, AND Z COORDINATES OF TRANSMITTER
C    POSITIONS.  Z INCREASES WITH DEPTH.
C   ********
C   IDPSR(IPSR)   POSRX(IPSR)   POSRY(IPSR)   POSRZ(IPSR)
C    (ONE LINE FOR EACH IPSR = 1 TO NPSR)
C   IDPSR(IPSR) IS THE IDENTIFICATION NUMBER OF THE RECEIVER POSITION.
C   POSRX, POSRY, AND POSRZ ARE X, Y, AND Z COORDINATES OF RECEIVER POSITIONS.
C   ********
C   NEXPTT       ADJTT
C
C   NEXPTT IS THE NUMBER OF EXPERIMENTAL TRAVEL TIMES.
C   ADJTT IS ADDED TO ALL EXPERIMENTAL TRAVEL TIMES.
C   ********
C   IF ITRMAX = -1, SKIP THIS INPUT
C   IDEXTT(IEXPTT)  IDEXTR(IEXPTT)  EXPTT(IEXPTT)    (WEIGHT(IEXPTT), IF MWT=1)
C    (ONE LINE FOR EACH IEXPTT = 1 TO NEXPTT)
C   IDEXTT AND IDEXTR ARE THE IDENTIFICATION NUMBERS OF THE TRANSMITTER AND
C    RECEIVER POSITIONS FOR THE TRAVEL TIME EXPTT(IEXPTT)
C   WEIGHT(IEXPTT) IS AN OPTIONAL WEIGHTING FACTOR FOR ACCURACY OF DATA.
C    CORRECTION FACTORS ARE MULTIPLIED BY THE CORRESPONDING WEIGHT. IF ALL DATA
C    ARE EQUALLY RELIABLE, SET EVERY WEIGHT = 1.0 OR SET MWT=0.
C   **************************************
C   IF MDATA = 1, USE THE FOLLOWING FORM:
C
C   NEXPTT      ADJDIS      ADJTT
C   *******
C   IDPATH(IEXPTT)  POSTX(IEXPTT)  POSTY(IEXPTT)  POSTZ(IEXPTT)  POSRX(IEXPTT)
C    POSRY(IEXPTT)  POSRZ(IEXPTT)  EXPTT(IEXPTT)    (WEIGHT(IEXPTT),IF MWT = 1)
C    (ONE LINE FOR EACH IEXPTT = 1 TO NEXPTT)
C   IDPATH(IEXPTT) IS THE IDENTIFICATION NUMBER OF A PATH AND
C    THE CORRESPONDING TRAVEL TIME. THEY DO NOT NEED TO BE SEQUENTIAL.
C   EXPTT(IEXPTT) IS THE EXPERIMENTAL TRAVEL TIME FOR PATH IEXPTT.
C  ****************************************************************************
C  SMFPLL   VLOCAL   VHICAL    VLODSP    VHIDSP    VLODIF    VHIDIF    INVOPT
C
C  SMFPLL IS THE LOWER LIMIT FOR THE SUM OF FRACTIONS OF PATH LENGTHS IN A
C   PIXEL (SEGMENT LENGTH IN PIXEL/LENGTH OF PATH FROM TRANS TO REC).  IF THE
C   SUM IS LESS THAN OR EQUAL SMFPLL, THEN THE VELOCITY FOR THE PIXEL IS
C   DISPLAYED AS ###.
C
C  VLOCAL AND VHICAL ARE THE LOW AND HIGH LIMITS FOR THE CALCULATED VELOCITIES.
C   IF A VELOCITY IS OUT OF THAT RANGE, IT IS SET = THE LIMIT.  IF NO LIMIT IS
C   DESIRED, SET THE LIMIT TO -99.
C
C  A COMPACT DISPLAY WITH VELOCITIES SCALED TO THE RANGE 0 TO 9 IS GENERATED,
C    AND A DOT MATRIX GRAPHICAL DISPLAY IS AN OPTION (SET IPGRA =1).
C  IF VLODSP (OR VHIDSP) IS -99., THEN THE LOWEST (OR HIGHEST) VELOCITY
C   DISPLAYED IS THE MINIMUM (OR MAXIMUM) CALCULATED BY BOMTOM.
C  IF VLODSP (OR VHIDSP) IS NOT - 99., THEN THE LOWEST (OR HIGHEST) VELOCITY
C   DISPLAYED IS VLODSP (OR VHIDSP).
C  WHEN DIFFERENCES BETWEEN THE VELOCITIES IN THIS RUN AND A PRIOR RUN ARE
C   DISPLAYED, VLODSP AND VHIDSP ARE REPLACED BY VLODIF AND VHIDIF.
C
C  INVOPT SPECIFIES THE OPTION FOR INPUTTING THE INITIAL GUESSES OF VELOCITIES.
C
C  IF INVOPT = 1, INPUT IS UNIFORM AND THE FORMAT IS:
C   V(1)
C   BOMTOM SETS INITIAL VELOCITY = V(1) FOR EVERY PIXEL.
C
C  IF INVOPT = 2, INPUT IS ROW BY ROW AND THE FORMAT IS:
C   IROWV     V(IROWV)                     (ONE LINE FOR EACH ROW, NPIXV LINES)
C   IROWV IS THE ROW NUMBER.  ROWS CAN BE INPUT IN ANY ORDER.
C   BOMTOM SETS INITIAL VELOCITIES = V(IROWV) FOR ALL PIXELS IN THAT ROW.
C    THIS OPTION FACILITATES VARYING THE PATTERN OF INITIAL GUESSES TO TEST THE
C    UNIQUENESS OF A SOLUTION.
C
C  IF INVOPT = 3, INPUT IS PIXEL BY PIXEL AND THE FORMAT IS:
C   IROWV   (V(I),I=1,NPIXH)              (ONE LINE FOR EACH ROW, NPIXV LINES)
C   IROWV IS THE ROW NUMBER.
C   V(I) IS THE INITIAL VELOCITY FOR ROW IROWV, COLUMN I.
C   THIS OPTION IS USED WHEN RESUMING A RUN USING A DATA FILE AS INPUT.
C*******END OF DESCRIPTION OF DATA READ FROM INPUT FILE************************
C******DEFINITIONS OF SOME OTHER VARIABLES USED IN BOMTOM**********************
C
C  DPIXH=DISTANCE FROM CENTER OF LEFT PIXEL TO CENTER OF RIGHT PIXEL.
C  DPIXV=DISTANCE FROM CENTER OF TOP PIXEL TO CENTER OF BOTTOM PIXEL.
C
C  POSTH AND POSRH ARE THE HORIZONTAL COORDINATES OF THE TRANSMITTER AND
C   RECEIVER IN THE COORDINATE SYSTEM OF THE PIXEL GRID.
C
C  ICOLT AND ICOLR ARE THE PIXEL COLUMNS CONTAINING POSTH AND POSRH.
C  IROWT AND IROWR ARE THE PIXEL ROWS CONTAINING POSTZ AND POSRZ.
C
C  NACROS IS THE NUMBER OF TIMES A PATH CROSSES PIXEL ROW BOUNDARY LINES.
C  TCROS(I), I=1,NACROS, IS THE HORIZONTAL DISTANCE FROM A TRANSMITTER
C   TO A POINT WHERE THE PATH CROSSES A PIXEL ROW BOUNDARY LINE.
C  ICOL(I) IS THE COLUMN IN WHICH CROSSING I OCCURS.
C  EDCROS(I) IS THE HORIZONTAL DISTANCE FROM THE LEFT EDGE OF ICOL(I)
C   TO CROSSING I.
C
C  D(J),J=1 TO BETWEEN SQRT(NPIXH**2+NPIXV**2) AND 2*SQRT(NPIXH**2+NPIXV**2),
C   IS THE LENGTH OF A PATH SEGMENT IN A PIXEL.  J DOES NOT DESIGNATE A
C   PARTICULAR PIXEL.
C  SMFPIX(IPIXEL),IPIXEL=1 TO NPIXH*NPIXV,IS THE SUM OF THE FRACTION OF PATH
C   LENGTHS IN PIXEL(IPIXEL).
C  IPIXSG(IPIXEL),IPIXEL=1 TO NPIXH*NPIXV, IS AN ARRAY KEEPING TRACK OF WHICH
C   PATH SEGMENTS ARE IN WHICH PIXELS.
C  LGLCOR(IPIXEL),IPIXEL=1 TO NPIXH*NPIXV, IS A LOGICAL ARRAY.  IF LGLCOR(I)
C   IS FALSE, THEN V(IPIXEL) IS NOT CHANGED.
C
C  DELT(I)=EXPTT(I)-CALCULATED TRAVEL TIME FOR PATH I.
C  RATIO(I)=EXPTT(I)/CALCULATED TRAVEL TIME FOR PATH I.
C*****************************************************************************
C*****************************************************************************
      IMPLICIT INTEGER*2(I-N)
      INTEGER*4 IARSG,IARSGS,NTOTSG,IOS
      CHARACTER*12 INFIL,OUTFIL,DATFIL,OLDFIL
      CHARACTER*79 HEADER,HEADF
      CHARACTER*6 VALPHA
      CHARACTER*1 VSCALE
      LOGICAL*1 LGLCOR
      LOGICAL FLXT
C
       DIMENSION VALPHA(65),VSCALE(65),  EDCROS(65),ICOL(65),TCROS(65),
     1 COR(4000),IPIXSG(4000),IVSCAL(4000),
     2  LGLCOR(4000),SMFPIX(4000),V(4000),VSMTH(4000),WTSMF(4000),
     3 DELT(1400),EXPTT(1400),EXTADJ(1400),IDEXTTT(1400),IDEXTTR(1400),
     4  IDPATH(1),RATIO(1400),WEIGHT(1),
     5 IDPST(60),POSTX(60),POSTY(60),POSTZ(60),
     6 IDPSR(60),POSRX(60),POSRY(60),POSRZ(60),
     7 D(150),IPP(1400),DPPATH(1400),DP(24000),IDPIX(24000)
C
C  IF ALL OPTIONS IN SECOND LINE OF INPUT ARE SET TO 0, THEN DIMENSIONS ARE:
C VALPHA(NPIXH),VSCALE(NPIXH),  EDCROS(NPIXV),ICOL(NPIXV),TCROS(NPIXV),
C COR(NPIXH*NPIXV),IPIXSG(NPIXH*NPIXV),IVSCAL(NPIXH*NPIXV),
C  LGLCOR(NPIXH*NPIXV),SMFPIX(NPIXH*NPIXV),V(NPIXH*NPIXV),VSMTH(1),WTSMF(1),
C DELT(NEXPTT),EXPTT(NEXPTT),EXTADJ(NEXPTT),IDEXTTT(NEXPTT),IDEXTTR(NEXPTT),
C  IDPATH(1),RATIO(NEXPTT),WEIGHT(1),
C IDPST(NPST),POSTX(NPST),POSTY(NPST),POSTZ(NPST),
C IDPSR(NPSR),POSRX(NPSR),POSRY(NPSR),POSRZ(NPSR),
C D(ISG), ISG IS LESS THAN 2*SQRT(NPIXH**2 + NPIXV**2) AND IS GIVEN IN OUTPUT.
C IPP(1),DPPATH(1),DP(1),IDPIX(1)
C
C  IF NOT ALL OPTIONS ARE SET TO 0, THEN ALTER DIMENSIONS AS FOLLOWS:
C IF MDATA=1, IDPATH(NEXPTT),POSTX(NEXPTT),POSTY(NEXPTT),POSTZ(NEXPTT),
C    POSRX(NEXPTT),POSRY(NEXPTT),POSRZ(NEXPTT)
C IF MWT=1, WEIGHT(NEXPTT),WTSMF(NPIXH*NPIXV)
C IF MSTOR=1, IPP(NEXPTT),DPPATH(NEXPTT),DP(IARSG),IDPIX(IARSG),
C  WHERE IARSG IS LESS THAN ISG*NEXPTT  AND IS GIVEN IN OUTPUT.
C IF MSMTH=1, VSMTH(NPIXH*NPIXV)
C
      DATA IYES,JYES /1HY,1Hy/
      DATA MDATA,MWT,MSTOR,MCOR,MBORE,MSMTH,MDIF/0,0,1,1,0,0,0/
      DATA ITRMAX,ITRSAV,TOLABS,TOLINC,RELAX/100,0,0.0,0.0,1.0/
      DATA IPDAT,IPIVG,IPFRC,IPERR,IPSEG,IPGRA,ISGRA/0,1,0,0,0,1,1/
      DATA SMFPLL,VLOCAL,VHICAL,VLODSP,VHIDSP,VLODIF,VHIDIF,INVOPT/
     1 0.0,-99.,-99.,-99.,-99.,-99.,-99.,3/
  100 MDFLG=0
      ICRFLG=0
  101 WRITE(*,102)
  102 FORMAT(' ENTER NAME OF EXISTING INPUT FILE OR OF NEW FILE TO BE CR
     1EATED: ')
      READ(*,103) INFIL
  103 FORMAT(A12)
      INQUIRE(FILE=INFIL,EXIST=FLXT,ERR=128,IOSTAT=IOS)
      IF(FLXT) THEN
       OPEN(3,FILE=INFIL,STATUS='OLD',ERR=128,IOSTAT=IOS)
       GO TO 123
      END IF
  104 WRITE(*,105) INFIL
  105 FORMAT(' CREATE A NEW INPUT FILE NAMED ',A12,' ? (Y/N): ')
      READ (*,124) MYN
      IF(MYN.NE.IYES .AND. MYN.NE.JYES) GO TO 101
      WRITE(*,107)
  107 FORMAT(' ENTER ONE LINE OF HEADING FOR INPUT FILE',/,1X)
      READ(*,306) HEADER
      WRITE(*,*)('A NEW INPUT FILE HAS BEEN STARTED, WITH OPTIONS SET FO
     1R NO CONSTRAINTS.')
      WRITE(*,108)
  108 FORMAT(/,' ENTER X,Y,Z COORDINATES OF CENTER OF TOP PIXEL ON TRANS
     1MITTER SIDE (POSP1X,     POSP1Y,POSP1Z). THEY ARE USUALLY THE SAME
     2 AS COORDINATES OF TOP TRANS POSITION  FOR SYMMETRIC TRANS, REC PO
     3SITIONS.',/,1X)
      READ(*,*) POSP1X,POSP1Y,POSP1Z
      WRITE(*,109)
  109 FORMAT(' ENTER X,Y,Z COORDINATES OF CENTER OF BOTTOM PIXEL ON RECE
     1IVER SIDE (POSPNX,     POSPNY,POSPNZ). THEY ARE USUALLY THE SAME A
     2S COORDINATES OF BOTTOM REC POSITION FOR SYMMETRIC TRANS, REC POSI
     3TIONS.',/,1X)
      READ(*,*) POSPNX,POSPNY,POSPNZ
      WRITE(*,110)
  110 FORMAT(' ENTER NUMBER OF PIXEL COLUMNS,ROWS (NPIXH,NPIXV).',/,' IF
     1 DESIRED PIXEL HEIGHT IS EPIXV, SET NPIXV=1+(POSPNZ-POSP1Z)/EPIXV.
     2',/,1X)
      READ(*,*) NPIXH,NPIXV
      NPXTOT=NPIXH*NPIXV
      WRITE(*,*)('ENTER NUM OF TRANS,REC POSITIONS (NPST,NPSR) ')
      READ(*,*) NPST,NPSR
      WRITE(*,111)
  111 FORMAT(' ENTER DISTANCE CORRECTION TO BE ADDED TO ALL RECEIVER POS
     1ITIONS (ADJDIS)',/,1X)
      READ(*,*) ADJDIS
      DO 113 IPST=1,NPST
      WRITE(*,112)
  112 FORMAT(' ENTER IDENT NUM, X,Y,Z COORDINATES OF TRANS POSITION (IDP
     1ST,POSTX,POSTY,POSTZ)',/,1X)
  113 READ(*,*) IDPST(IPST),POSTX(IPST),POSTY(IPST),POSTZ(IPST)
      DO 115 IPSR=1,NPSR
      WRITE(*,114)
  114 FORMAT(' ENTER IDENT NUM, X,Y,Z, COORDINATES OF REC POSITION (IDPS
     1R,POSRX,POSRY,POSRZ)',/,1X)
  115 READ(*,*) IDPSR(IPSR),POSRX(IPSR),POSRY(IPSR),POSRZ(IPSR)
      WRITE(*,*)('ENTER NUM OF TRAVEL TIMES (NEXPTT) ')
      READ(*,*) NEXPTT
      WRITE(*,116)
  116 FORMAT(' ENTER TIME CORRECTION TO BE ADDED TO EACH TRAVEL TIME (AD
     1JTT)',/,1X)
      READ(*,*) ADJTT
      DO 118 IEXPTT=1,NEXPTT
      WRITE(*,117)
  117 FORMAT(' ENTER TRANS IDENT NUM, REC IDENT NUM, TRAVEL TIME (IDEXTT
     1T,IDEXTTR,EXPTT)',/,1X)
  118 READ(*,*) IDEXTTT(IEXPTT),IDEXTTR(IEXPTT),EXPTT(IEXPTT)
      WRITE(*,*)('ENTER GUESS FOR UNIFORM INITIAL VELOCITY (V) ')
      READ(*,*) V(1)
      DO 119 IPIXEL=2,NPXTOT
  119 V(IPIXEL)=V(1)
      OPEN(4,FILE=INFIL,STATUS='NEW',ERR=128,IOSTAT=IOS)
      ICRFLG=-99
      GO TO 805
  120 WRITE(*,121) INFIL
  121 FORMAT(1X,A12,' HAS BEEN CREATED. RUN PROGRAM ? (Y/N): ')
      READ (*,124) MYN
      IF(MYN.NE.IYES .AND. MYN.NE.JYES) THEN
       STOP
      END IF
      ICRFLG=0
      OPEN(3,FILE=INFIL,STATUS='OLD',ERR=128,IOSTAT=IOS)
  123 WRITE(*,*)('ENTER NAME OF OUTPUT FILE FOR PRINTING: ')
      READ(*,103) OUTFIL
      INQUIRE(FILE=OUTFIL,EXIST=FLXT,ERR=128,IOSTAT=IOS)
      IF(.NOT.FLXT) GO TO 125
      WRITE(*,*)('FILE ALREADY EXISTS.  WRITE OVER IT?  (Y/N): ')
      READ (*,124) MYN
  124 FORMAT(A1)
      IF(MYN.NE.IYES .AND. MYN.NE.JYES) GO TO 123
  125 OPEN(2,FILE=OUTFIL,ERR=128,IOSTAT=IOS)
  126 WRITE(*,*)('ENTER NAME OF DATA FILE FOR SAVING RESULTS: ')
      READ(*,103) DATFIL
      INQUIRE(FILE=DATFIL,EXIST=FLXT,ERR=128,IOSTAT=IOS)
      IF(.NOT.FLXT) GO TO 130
      WRITE(*,*)('FILE ALREADY EXISTS.  WRITE OVER IT?  (Y/N): ')
      READ (*,124) MYN
      IF(MYN.NE.IYES .AND. MYN.NE.JYES) GO TO 126
      GO TO 130
  128 WRITE(*,129) IOS
  129 FORMAT(/,' ERROR #',I5,' IN INQUIRE OR OPEN STATEMENT.')
      GO TO 101
C
C******START READING INPUT FILE, LET OPERATOR EXAMINE SOME LINES*******
  130 READ(3,132) HEADER
      IF(MDFLG.NE.-999) WRITE(*,132) HEADER
  132 FORMAT(1X,A79)
      READ(3,*) MDATA,MWT,MSTOR,MCOR,MBORE,MSMTH,MDIF
      IF(MDFLG.NE.-999)WRITE(*,134)MDATA,MWT,MSTOR,MCOR,MBORE,MSMTH,MDIF
  134 FORMAT(' MDATA=',I1,'   MWT=',I1,'   MSTOR=',I1,
     1'   MCOR=',I1,'   MBORE=',I1,'   MSMTH=',I1,'    MDIF=',I1)
      READ(3,*) ITRMAX,ITRSAV,TOLABS,TOLINC,RELAX
      IF(MDFLG.NE.-999) WRITE(*,136) ITRMAX,ITRSAV,TOLABS,TOLINC,RELAX
  136 FORMAT(' ITRMAX=',I5,'  ITRSAV=',I5,'  TOLABS=',F10.7,
     1'  TOLINC=',F10.7,'  RELAX=',F7.3)
      READ(3,*) IPDAT,IPIVG,IPFRC,IPERR,IPSEG,IPGRA,ISGRA
      READ(3,*) POSP1X,POSP1Y,POSP1Z,POSPNX,POSPNY,POSPNZ
      READ(3,*) NPIXH,NPIXV,NLITOP,NLIBOT,NFXLFT,NFXRHT
      NPXTOT=NPIXH*NPIXV
       IF(MDATA.NE.1 .OR.ITRMAX.EQ.-1) THEN
       READ(3,*) NPST,NPSR,ADJDIS
       DO 138 IPST=1,NPST
  138  READ(3,*) IDPST(IPST),POSTX(IPST),POSTY(IPST),POSTZ(IPST)
       DO 140 IPSR=1,NPSR
  140  READ(3,*) IDPSR(IPSR),POSRX(IPSR),POSRY(IPSR),POSRZ(IPSR)
       READ(3,*) NEXPTT,ADJTT
        IF(ITRMAX.EQ.-1) THEN
        DO 141 IPST=1,NPST
        DO 141 IPSR=1,NPSR
        IDEXTTT((IPST-1)*NPSR+IPSR)=IDPST(IPST)
  141   IDEXTTR((IPST-1)*NPSR+IPSR)=IDPSR(IPSR)
        GO TO 150
        END IF
        IF(MWT.NE.1) THEN
        DO 142 IEXPTT=1,NEXPTT
  142   READ(3,*) IDEXTTT(IEXPTT),IDEXTTR(IEXPTT),EXPTT(IEXPTT)
        ELSE
        DO 144 IEXPTT=1,NEXPTT
  144   READ(3,*) IDEXTTT(IEXPTT),IDEXTTR(IEXPTT),EXPTT(IEXPTT),
     1   WEIGHT(IEXPTT)
        END IF
       ELSE
       READ(3,*) NEXPTT,ADJDIS,ADJTT
       IF(MWT.NE.1) THEN
       DO 146 IEXPTT=1,NEXPTT
  146  READ(3,*)IDPATH(IEXPTT),POSTX(IEXPTT),POSTY(IEXPTT),POSTZ(IEXPTT)
     1,POSRX(IEXPTT),POSRY(IEXPTT),POSRZ(IEXPTT),EXPTT(IEXPTT)
        ELSE
       DO 148 IEXPTT=1,NEXPTT
  148  READ(3,*)IDPATH(IEXPTT),POSTX(IEXPTT),POSTY(IEXPTT),POSTZ(IEXPTT)
     1,POSRX(IEXPTT),POSRY(IEXPTT),POSRZ(IEXPTT),EXPTT(IEXPTT),
     2 WEIGHT(IEXPTT)
        END IF
       END IF
  150 READ(3,*) SMFPLL,VLOCAL,VHICAL,VLODSP,VHIDSP,VLODIF,VHIDIF,INVOPT
      GO TO (156,160,164)INVOPT
      WRITE(*,154)
      WRITE(2,154)
  154 FORMAT(' INVOPT SPECIFIED INCORRECTLY, PROGRAM TERMINATED.',/,
     1 ' THIS ERROR IS USUALLY CAUSED BY THE WRONG NUMBER OF INPUT',/,
     2'   PARAMETERS SOMEWHERE BEFORE INVOPT IS READ.')
      GO TO 999
  156 READ(3,*) V(1)
       DO 158 IPIXEL=2,NPXTOT
  158  V(IPIXEL) = V(1)
      GO TO 168
  160  DO 162 IPIXV=1,NPIXV
       READ(3,*) IROWV, V(1+(IROWV-1)*NPIXH)
        DO 162 IPIXH=2,NPIXH
  162   V(IPIXH+(IROWV-1)*NPIXH) = V(1+(IROWV-1)*NPIXH)
      GO TO 168
  164  DO 166 IPIXV=1,NPIXV
  166  READ(3,*)IROWV,(V(IPIXH+(IROWV-1)*NPIXH),IPIXH=1,NPIXH)
  168 CLOSE(3)
      IF(MDFLG.EQ.-999) GO TO 992
C***********END OF READING INPUT FILE******************************************
C***********START MODIFYING INPUT USING KEYBOARD, IF DESIRED.*****************
  170 NPXTOT=NPIXH*NPIXV
      DFACX=POSPNX-POSP1X
      DFACY=POSPNY-POSP1Y
      DPIXHS=(DFACX*DFACX + DFACY*DFACY)
      DPIXH=SQRT(DPIXHS)
      DPIXV=POSPNZ-POSP1Z
      EPIXH=DPIXH/(NPIXH-1)
      EPIXV=DPIXV/(NPIXV-1)
      WRITE(*,308) HEADER
      WRITE(*,*)(' CHANGE INPUT?  RUN PROGRAM OR ENTER NUMBER OF ANY OPT
     1ION YOU WANT TO CHANGE.')
      WRITE(*,*)('NUM  OPTION')
      WRITE(*,*)(' 1   RUN PROGRAM')
      IF(MSTOR.NE.1)WRITE(*,*)(' 2   MSTOR = 0, RECALCULATE PATH SEGMENT
     1S EACH ITERATION')
      IF(MSTOR.EQ.1)WRITE(*,*)(' 2   MSTOR = 1, STORE AND RECALL PATH SE
     1GMENTS')
      IF(MBORE.NE.1)WRITE(*,*)(' 3   MBORE = 0, VEL AT TRANS AND REC POS
     1 ARE VARIED UNLESS FIXED BY OPT 12')
      IF(MBORE.EQ.1)WRITE(*,*)(' 3   MBORE = 1, VELOCITIES AT TRANS AND
     1 REC POS ARE FIXED')
      IF(MSMTH.NE.1) WRITE(*,*)(' 4   MSMTH = 0, NO SMOOTHING')
      IF(MSMTH.EQ.1) WRITE(*,*)(' 4   MSMTH = 1, PERFORM SMOOTHING')
      IF(MDIF.NE.1)WRITE(*,*)(' 5   MDIF = 0, WILL NOT BE COMPARED WITH
     1 PRIOR RUN')
      IF(MDIF.EQ.1)WRITE(*,*)(' 5   MDIF = 1, WILL BE COMPARED WITH PRIO
     1R RUN')
      WRITE(*,173) ITRMAX,TOLABS,TOLINC,RELAX,IPDAT,IPIVG,IPFRC,IPERR,
     1IPSEG,IPGRA,ISGRA,NPIXH,NPIXV,EPIXH,EPIXV,NLITOP,NLIBOT,NFXLFT,
     2NFXRHT,ADJDIS,ADJTT,SMFPLL
  173 FORMAT('  6   ITRMAX (ITERATION LIMIT) = ',I6,/,    '  7   TOLABS,
     1TOLINC (ABSOLUTE,INCREMENTAL ERROR TOL FOR CONV) = ',F7.4,F8.4,
     2/,'  8   RELAX (RELAXATION PARAMETER) = ',F9.5,/,'  9   IPDAT,IPIV
     3G,IPFRC,IPERR,IPSEG,IPGRA,ISGRA (1 OUTPUT: DATA,INITIAL',/,'
     4 VEL,PATH FRAC,ERRORS,SEG LENGTHS,VEL GRAPH,SCRN VEL GRAPH)= ',I1,
     56I2,/,' 10   NPIXH,NPIXV (NUMBER OF COLUMNS,ROWS OF PIXELS) = ',2I
     64,/,'          (PIXEL WIDTH,HEIGHT =',2F8.3,')',/,
     7    ' 11   NLITOP,NLIBOT (NUM OF LATERALLY INVARIANT ROWS AT TOP,B
     8OTTOM) =',2I3,/,' 12   NFXLFT,NFXRHT (NUM OF COL WITH FIXED VEL AT
     8 LEFT,RIGHT SIDES) =',2I3,/,' 13   ADJDIS,ADJTT (DISTANCE, TIME AD
     9DED) = ',2F10.4,/,' 14   SMFPLL (LOWER LIMIT OF PATH FRACTIONS FOR
     9 DISPLAY) = ',F8.4)
      WRITE(*,174) VLOCAL,VHICAL
  174 FORMAT(' 15   VLOCAL,VHICAL (LIMITS OF CALC VELOCITY,UNLESS = -99.
     1) =',2F9.3)
      WRITE(*,175) VLODSP,VHIDSP
  175 FORMAT(' 16   VLODSP,VHIDSP (VELOCITY LIMITS IN GRAPHS AND SCALED
     1DISPLAYS',/,'       UNLESS = -99., THEN LIMITS ARE MIN,MAX VELOCIT
     1IES) =',2F9.3)
      WRITE(*,176) VLODIF,VHIDIF
  176 FORMAT(' 17   VLODIF,VHIDIF (VELOCITY DIFFERENCE LIMITS IN GRAPHS)
     1 =',2F9.3)
      WRITE(*,177) INVOPT
  177 FORMAT(' 18   INVOPT (OPTION FOR INPUT OF INITIAL VELOCITY GUESS:
     1 1 FOR',/,'             UNIFORM, 2 FOR ROW BY ROW, 3 FOR PIXEL BY
     2PIXEL) = ',I2)
      WRITE(*,*)('ENTER OPTION NUMBER. ')

      READ(*,*)IFLG
      GO TO(280,180,185,190,195,200,205,210,215,220,225,228,230,235,
     1 238,240,243,245) IFLG
      WRITE(*,*) (' INCORRECT OPTION NUMBER.  REENTER OPTION, 1 TO 18')
      GO TO 170
  180 IF(MSTOR.EQ.1) MTEMP=0
      IF(MSTOR.NE.1) MTEMP=1
      MSTOR=MTEMP
      GO TO 170
  185 IF(MBORE.EQ.1) MTEMP=0
      IF(MBORE.NE.1) MTEMP=1
      MBORE=MTEMP
      GO TO 170
  190 IF(MSMTH.EQ.1) MTEMP=0
      IF(MSMTH.NE.1) MTEMP=1
      MSMTH=MTEMP
      GO TO 170
  195 IF(MDIF.EQ.1) MTEMP=0
      IF(MDIF.NE.1) MTEMP=1
      MDIF=MTEMP
      GO TO 170
  200 WRITE(*,202) ITRMAX
  202 FORMAT(' CURRENT ITRMAX IS',I5,/,'  ENTER NEW ITRMAX  ')
      READ(*,*) ITRMAX
      GO TO 170
  205 WRITE(*,207) TOLABS,TOLINC
  207 FORMAT(' CURRENT TOLABS,TOLINC ARE',2F8.4,/,
     1'   ENTER NEW TOLABS,TOLINC ')
      READ(*,*) TOLABS,TOLINC
      GO TO 170
  210 WRITE(*,212) RELAX
  212 FORMAT(' CURRENT RELAX IS',F10.6,/,'  ENTER NEW RELAX ')
      READ(*,*) RELAX
      GO TO 170
  215 WRITE(*,217) IPDAT,IPIVG,IPFRC,IPERR,IPSEG,IPGRA,ISGRA
  217 FORMAT(' CURRENT IPDAT,IPIVG,IPFRC,IPERR,IPSEG,IPGRA,ISGRA ARE',
     1 7I2,/,'   ENTER NEW IPDAT,IPIVG,IPFRC,IPERR,IPSEG,IPGRA,ISGRA ')
      READ(*,*)IPDAT,IPIVG,IPFRC,IPERR,IPSEG,IPGRA,ISGRA
      GO TO 170
  220 WRITE(*,222) NPIXH,NPIXV
  222 FORMAT(' CURRENT NPIXH,NPIXV ARE',2I4,/,
     1 '   ENTER NEW NPIXH,NPIXV ')
      READ(*,*) NPIXH,NPIXV
      IF (INVOPT.EQ.1) GO TO 255
      WRITE(*,*)('IF NPIXH,NPIXV ARE CHANGED AND INVOPT=2 OR 3, BE SURE
     1INITIAL VEL GUESSES ARE   FOR CURRENT PIXEL GRID SIZE')
      GO TO 245
  225 WRITE(*,227) NLITOP,NLIBOT
  227 FORMAT(' CURRENT NLITOP,NLIBOT ARE',2I3,/,
     1 '   ENTER NEW NLITOP,NLIBOT ')
      READ(*,*) NLITOP,NLIBOT
      GO TO 170
  228 WRITE(*,229) NFXLFT,NFXRHT
  229 FORMAT(' CURRENT NFXLFT,NFXRHT ARE',2I3,/,
     1 '   ENTER NEW NFXLFT,NFXRHT ')
      READ(*,*) NFXLFT,NFXRHT
      GO TO 170
  230 WRITE(*,232) ADJDIS,ADJTT
  232 FORMAT(' CURRENT ADJDIS,ADJTT ARE',2F9.4,/,
     1'   ENTER NEW ADJDIS,ADJTT ')
      READ(*,*) ADJDIS,ADJTT
      GO TO 170
  235 WRITE(*,237) SMFPLL
  237 FORMAT(' CURRENT SMFPLL IS',F7.4,/,'  ENTER NEW SMFPLL ')
      READ(*,*) SMFPLL
      GO TO 170
  238 WRITE(*,239) VLOCAL,VHICAL
  239 FORMAT(' CURRENT VLOCAL,VHICAL ARE',2F10.4,/,
     1 '   ENTER NEW VLOCAL,VHICAL  ')
      READ(*,*) VLOCAL,VHICAL
      GO TO 170
  240 WRITE(*,242) VLODSP,VHIDSP
  242 FORMAT(' CURRENT VLODSP,VHIDSP ARE',2F10.4,/,
     1 '   ENTER NEW VLODSP,VHIDSP  ')
      READ(*,*) VLODSP,VHIDSP
      GO TO 170
  243 WRITE(*,244) VLODIF,VHIDIF
  244 FORMAT(' CURRENT VLODIF,VHIDIF ARE',2F10.4,/,
     1 '   ENTER NEW VLODIF,VHIDIF  ')
      READ(*,*) VLODIF,VHIDIF
      GO TO 170
  245 WRITE(*,247) INVOPT
  247 FORMAT(' CURRENT INVOPT IS ',I3,/,' ENTER NEW INVOPT (1 UNIFORM; 2
     1 ROW BY ROW; OR 3 PIXEL BY PIXEL) ')
      READ(*,*) INVOPT
      GO TO (252,258,264) INVOPT
      WRITE(*,250)
  250 FORMAT(' INVOPT NOT SPECIFIED CORRECTLY.  TRY AGAIN.')
      GO TO 245
  252 WRITE(*,254) V(1)
  254 FORMAT(' CURRENT INITIAL GUESS IS ',F8.4,/,'  ENTER NEW INITIAL GU
     1ESS ')
      READ(*,*) V(1)
  255  DO 256  IV=2,NPIXH*NPIXV
  256  V(IV) = V(1)
      GO TO 170
  258 WRITE(*,*)(' ENTER PIXEL ROW NUMBER. NEG ENTRY ENDS THIS INPUT ')
      READ(*,*) IPIXV
      IF(IPIXV.LT.0) GO TO 170
      WRITE(*,260) V(1+(IPIXV-1)*NPIXH)
  260 FORMAT(' CURRENT INITIAL GUESS IS ',F8.4,/,'  ENTER NEW INITIAL GU
     1ESS ')
      READ(*,*) VTR
       DO 262 IPIXH=1,NPIXH
  262  V(IPIXH+(IPIXV-1)*NPIXH)=VTR
      GO TO 258
  264 WRITE(*,*)(' ENTER PIXEL ROW NUMBER. NEG ENTRY ENDS THIS INPUT ')
       READ(*,*) IPIXV
       IF(IPIXV.LT.0) GO TO 170
  266 WRITE(*,*)(' ENTER PIXEL COLUMN NUMBER ')
      READ(*,*) IPIXH
      WRITE(*,268) V(IPIXH+(IPIXV-1)*NPIXH)
  268 FORMAT(' CURRENT INITIAL GUESS IS ',F8.4,/,'  ENTER NEW INITIAL GU
     1ESS ')
      READ(*,*) V(IPIXH+(IPIXV-1)*NPIXH)
      GO TO 264
C***********END OF MODIFYING INPUT USING KEYBOARD*****************************
C*******CHECK FILE FOR PRIOR RUN IF COMPARISONS WILL BE MADE******************
  280 IF (MDIF.NE.1 .OR. MDFLG.EQ.-999) GO TO 300
  282 WRITE(*,283)
  283 FORMAT(' ENTER NAME OF FILE OF PRIOR RUN FOR COMPARING VELOCITIES:
     1 ')
      READ(*,284) OLDFIL
  284 FORMAT(A12)
      INQUIRE(FILE=OLDFIL,EXIST=FLXT,ERR=128,IOSTAT=IOS)
      IF(FLXT) GO TO 289
  286 WRITE(*,288)
  288 FORMAT(' FILE FOR PRIOR RUN WAS NOT FOUND.',/,
     1 ' CHECK SPELLING, EXTENSION, AND PATH.  THEN TRY AGAIN.')
      GO TO 282
  289 OPEN(3,FILE=OLDFIL,STATUS='OLD',ERR=286,IOSTAT=IOS)
  290 READ(3,132) HEADF
      WRITE(*,132) HEADF
      READ(3,*) MDADF,MWTDF,MSTRDF,MCRDF,MBRDF,MSMTDF,MDIFDF
      WRITE(*,134) MDADF,MWTDF,MSTRDF,MCRDF,MBRDF,MSMTDF,MDIFDF
      READ(3,*) ITMDF,ITSDF,TLADF,TLIDF,RELDF
      WRITE(*,136) ITMDF,ITSDF,TLADF,TLIDF,RELDF
      CLOSE(3)
      WRITE(*,*)(' IS THIS THE CORRECT FILE FOR PRIOR RUN?  (Y/N): ')
      READ(*,292) MYN
  292 FORMAT(A1)
      IF(MYN.NE.IYES .AND. MYN.NE.JYES) GO TO 282
C***********PRINT INPUT PARAMETERS********************************************
  300 WRITE(2,302)
  302 FORMAT(1H1)
      WRITE(*,304)
  304 FORMAT(' ENTER ONE LINE OF HEADING FOR OUTPUT',/,1X)
      READ(*,306) HEADER
  306 FORMAT(A79)
      WRITE(2,308) HEADER
  308 FORMAT(1X,A79)
      IF(ITRMAX.EQ.-1) THEN
      WRITE(2,*)('SYNTHETIC TRAVEL TIME GENERATION FROM BOMTOM')
      GO TO 309
      END IF
      IF(MDFLG.NE.-999) WRITE(2,*)('TOMOGRAPHIC RECONSTRUCTION FROM BOMT
     1OM')
      IF(MDFLG.EQ.-999) WRITE(2,*)('PARAMETERS OF PRIOR RUN USED FOR COM
     6PARISON')
  309 WRITE(2,310) INFIL
  310 FORMAT(/,' INPUT FILE IS ',A12)
      WRITE(2,312) OUTFIL
  312 FORMAT(' THIS OUTPUT FILE IS ',A12)
      WRITE(2,313) DATFIL
  313 FORMAT(' OUTPUT DATA FILE IS ',A12)
      IF(MDATA.NE.1) WRITE(2,*)('MDATA = 0, POSITIONS AND TIMES ARE INPU
     1T SEPARATELY')
      IF(MDATA.EQ.1) WRITE(2,*)('MDATA = 1, POSITIONS AND TIMES ARE INPU
     1T TOGETHER')
      IF(MWT.NE.1) WRITE(2,*)('MWT = 0, NO WEIGHTING FACTORS FOR ACCURAC
     1Y OF TRAVEL TIMES')
      IF(MWT.EQ.1) WRITE(2,*)('MWT = 1, USE WEIGHTING FACTORS FOR ACCURA
     1CY OF TRAVEL TIMES')
      IF(MSTOR.NE.1)WRITE(2,*)('MSTOR = 0, RECALCULATE PATH SEGMENTS')
      IF(MSTOR.EQ.1)WRITE(2,*)('MSTOR = 1, STORE AND RECALL PATH SEGMENT
     1S')
      IF(MCOR.NE.1)WRITE(2,*)('MCOR = 0, MULTIPLICATIVE CORRECTION FACTO
     1RS')
      IF(MCOR.EQ.1)WRITE(2,*)('MCOR = 1, ADDITIVE CORRECTION FACTORS')
      IF(MBORE.NE.1)WRITE(2,*)('MBORE = 0, VELOCITIES AT TRANS AND REC P
     1OS ARE VARIED UNLESS COL VEL FIXED')
      IF(MBORE.EQ.1)WRITE(2,*)('MBORE = 1, VELOCITIES AT TRANS AND REC P
     1OS ARE FIXED')
      IF(MSMTH.NE.1) WRITE(2,*)('MSMTH = 0, NO SMOOTHING')
      IF(MSMTH.EQ.1) WRITE(2,*)('MSMTH = 1, PERFORM SMOOTHING')
      IF(MDIF.NE.1)WRITE(2,*)('MDIF = 0, WILL NOT BE COMPARED WITH PRIOR
     1 RUN')
      IF(MDIF.EQ.1) WRITE(2,314) OLDFIL
  314 FORMAT(' MDIF = 1, WILL BE COMPARED WITH ',A12)
      WRITE(2,316) IPDAT,IPIVG,IPFRC,IPERR,IPSEG,IPGRA,ISGRA
  316 FORMAT(' IPDAT,IPIVG,IPFRC,IPERR,IPSEG,IPGRA,ISGRA = ',I2,',',I2,
     1 ',',I2,',',I2,',',I2,',',I2,',',I2)
      WRITE(2,320)NEXPTT,POSP1X,POSP1Y,POSP1Z,POSPNX,POSPNY,POSPNZ,
     1 NPIXH,NPIXV,EPIXH,EPIXV,NLITOP,NLIBOT,NFXLFT,NFXRHT,TOLABS,
     2 TOLINC,RELAX
  320 FORMAT(/,' NUMBER OF EXPERIMENTAL TRAVEL TIMES = ',I4,/,
     1' CENTER OF PIXEL 1 IS AT X =',F10.4,'  Y =',F10.4,'  Z =',F10.4,
     2/,' CENTER OF LAST PIXEL IS AT X =',F10.4,'  Y =',F10.4,'  Z =',
     3 F10.4,/,' NUMBER OF COLUMNS, ROWS OF PIXELS = ',I3,',',I3,/,
     4' PIXEL WIDTH, HEIGHT = ',F9.4,',',F9.4,/,
     5' NUMBER OF LATERALLY INVARIANT ROWS AT TOP, BOTTOM = ',I2,',',I2,
     6/,' NUMBER OF FIXED VELOCITY COLUMNS AT LEFT, RIGHT = ',I2,',',I2,
     7  /,' ABSOLUTE ERROR TOLERANCE FOR CONVERGENCE = ',F9.6,
     8  /,' INCREMENTAL ERROR TOLERANCE FOR CONVERGENCE = ',F9.6,
     9       /,' RELAXATION PARAMETER =',F9.5)
      WRITE(2,322) ADJDIS,ADJTT
  322 FORMAT(' DISTANCE ADJUSTMENT ADDED BETWEEN BOREHOLES = ',F10.4,/,
     1 ' TIME ADJUSTMENT ADDED TO ALL TRAVEL TIMES = ',F10.4)
      IF(VLOCAL.EQ.-99. .AND. VHICAL.EQ.-99.) WRITE(2,*)
     6 ('NO LIMITS ON CALCULATED VELOCITIES.')
      IF(VLOCAL.NE.-99.) WRITE(2,323) VLOCAL
  323 FORMAT(' LOWEST ALLOWED VELOCITY =',F8.3)
      IF(VHICAL.NE.-99.) WRITE(2,324) VHICAL
  324 FORMAT(' HIGHEST ALLOWED VELOCITY =',F8.3)
      IF(ITRMAX.EQ.-1)GO TO 400
  328 IF(IPDAT.NE.1)GO TO 365
       IF(MDATA.NE.1) THEN
       WRITE(2,330) NPST,NPSR
  330  FORMAT(//,' NUMBER OF TRANSMITTER, RECEIVER POSITIONS =',I4,',',
     1 I4)
       WRITE(2,*)('POSITIONS LISTED HAVE NOT BEEN ADJUSTED BY ADJDIS')
       WRITE(2,332)
  332  FORMAT(/,' TRANS IDENT        X              Y              Z')
       DO 334 IPST=1,NPST
  334  WRITE(2,336) IDPST(IPST),POSTX(IPST),POSTY(IPST),POSTZ(IPST)
  336  FORMAT(I10,3F15.4)
       WRITE(2,338)
  338  FORMAT(/,'   REC IDENT        X              Y              Z')
       DO 340 IPSR=1,NPSR
  340  WRITE(2,336) IDPSR(IPSR),POSRX(IPSR),POSRY(IPSR),POSRZ(IPSR)
       WRITE(2,342) NEXPTT
  342  FORMAT(//,' NUMBER OF TRAVEL TIMES =',I6)
       WRITE(2,344)
  344  FORMAT(' TRAVEL TIMES LISTED HAVE NOT BEEN ADJUSTED BY ADJTT',/)
        IF(MWT.NE.1) THEN
        WRITE(2,*)('TRANS ID   REC ID    TRAVEL TIME        TRANS ID   R
     1EC ID    TRAVEL TIME')
        NEXP2=NEXPTT/2
        DO 346 IEXP2=1,NEXP2
        ISUM2=IEXP2+NEXP2
  346   WRITE(2,348) IDEXTTT(IEXP2),IDEXTTR(IEXP2),EXPTT(IEXP2),
     1   IDEXTTT(ISUM2),IDEXTTR(ISUM2),EXPTT(ISUM2)
  348   FORMAT(1X,I7,I9,F15.5,9X,I7,I9,F15.5)
        IF(ISUM2.LT.NEXPTT)WRITE(2,349) IDEXTTT(NEXPTT),IDEXTTR(NEXPTT),
     1   EXPTT(NEXPTT)
  349   FORMAT(41X,I7,I9,F15.5)
        ELSE
        WRITE(2,*)('TRANS IDENT   REC IDENT      TRAVEL TIME  WEIGHT')
        DO 350 IEXPTT=1,NEXPTT
  350   WRITE(2,352) IDEXTTT(IEXPTT),IDEXTTR(IEXPTT),EXPTT(IEXPTT),
     1  WEIGHT(IEXPTT)
  352   FORMAT(1X,I9,I13,F18.5,F7.3)
        END IF
       ELSE
       WRITE(2,354)
  354  FORMAT(//,' POSITIONS AND TRAVEL TIMES LISTED HAVE NOT BEEN ADJUS
     1TED BY ADJDIS AND ADJTT')
        IF(MWT.NE.1) THEN
       WRITE(2,356)
  356  FORMAT(/,'PATH ID  TRANS X   TRANS Y   TRANS Z    REC X    REC Y
     1 REC Z   TRAVEL TIME')
       DO 358 IEXPTT=1,NEXPTT
  358  WRITE(2,360)IDPATH(IEXPTT),POSTX(IEXPTT),POSTY(IEXPTT),
     1 POSTZ(IEXPTT),POSRX(IEXPTT),POSRY(IEXPTT),POSRZ(IEXPTT),
     2 EXPTT(IEXPTT)
  360  FORMAT(1X,I4,3F10.4,3F10.4,F9.5)
        ELSE
       WRITE(2,361)
  361  FORMAT(/,' PATH   TRANS X   TRANS Y   TRANS Z    REC X     REC Y
     1   REC Z  TRAV TIME  WT')
       DO 362 IEXPTT=1,NEXPTT
  362  WRITE(2,363)IDPATH(IEXPTT),POSTX(IEXPTT),POSTY(IEXPTT),
     1 POSTZ(IEXPTT),POSRX(IEXPTT),POSRY(IEXPTT),POSRZ(IEXPTT),
     2 EXPTT(IEXPTT),WEIGHT(IEXPTT)
  363  FORMAT(1X,I4,3F10.4,3F10.4,F9.5,F5.2)
        END IF
       END IF
  365 IF(IPIVG.NE.1) GO TO 369
      WRITE(2,366)
  366 FORMAT(/,' INITIAL GUESSES FOR VELOCITIES FOR EACH PIXEL')
      DO 367 IPIXV=1,NPIXV
  367 WRITE(2,368) (V(IPIXH+(IPIXV-1)*NPIXH),IPIXH=1,NPIXH)
  368 FORMAT(1X,80F6.2)
  369 IF(IPGRA.NE.1) GO TO 395
      IF(VLODSP.EQ.-99.) THEN
        VMIN=10000.
        DO 370 IPIXEL=1,NPXTOT
        VMIN=AMIN1(VMIN,V(IPIXEL))
  370   CONTINUE
      ELSE
        VMIN=VLODSP
      END IF
      IF(VHIDSP.EQ.-99.) THEN
        VMAX=-10000.
        DO 371 IPIXEL=1,NPXTOT
        VMAX=AMAX1(VMAX,V(IPIXEL))
  371   CONTINUE
      ELSE
        VMAX=VHIDSP
      END IF
      VDIFF=VMAX-VMIN
      DIV1=VMIN+0.2*VDIFF
      DIV2=VMIN+0.4*VDIFF
      DIV3=VMIN+0.6*VDIFF
      DIV4=VMIN+0.8*VDIFF
      WRITE(2,981)
      WRITE(2,*)('  GRAPH OF INITIAL VELOCITY GUESSES')
      WRITE(2,*)(' VELOCITY LIMITS FOR A PATTERN ARE ON EACH SIDE OF IT.
     1')
      WRITE(2,982) VMIN,CHAR(32),CHAR(32),DIV1,CHAR(176),CHAR(176),
     1 DIV2,CHAR(177),CHAR(177),DIV3,CHAR(178),CHAR(178),DIV4,CHAR(219),
     2 CHAR(219),VMAX
      DO 390 IPIXV=1,NPIXV
      DO 384 IPIXH=1,NPIXH
      IPIXEL=IPIXH+(IPIXV-1)*NPIXH
        IF(V(IPIXEL).LT.VMIN) IASCI=60
        IF(V(IPIXEL).GE.VMIN .AND. V(IPIXEL).LT.DIV1) IASCI=32
        IF(IASCI.EQ.32 .AND. IPIXV.EQ.1 .AND. IPIXH.EQ.2) IASCI=196
        IF(V(IPIXEL).GE.DIV1 .AND. V(IPIXEL).LT.DIV2) IASCI=176
        IF(V(IPIXEL).GE.DIV2 .AND. V(IPIXEL).LE.DIV3) IASCI=177
        IF(V(IPIXEL).GT.DIV3 .AND. V(IPIXEL).LE.DIV4) IASCI=178
        IF(V(IPIXEL).GT.DIV4 .AND. V(IPIXEL).LE.VMAX) IASCI=219
        IF(V(IPIXEL).GT.VMAX) IASCI=62
         IF(IPIXV.EQ.1 .AND. IASCI.EQ.32) IASCI=196
         IF(IPIXV.EQ.NPIXV .AND. IASCI.EQ.32) IASCI=95
        WRITE(VSCALE(IPIXH),985) CHAR(IASCI)
  384 CONTINUE
      IF(NPIXH.LT.40) WRITE(2,987) (VSCALE(IPIXH),VSCALE(IPIXH),
     1 IPIXH=1,NPIXH)
      IF(NPIXH.GE.40) WRITE(2,987) (VSCALE(IPIXH),IPIXH=1,NPIXH)
  390 CONTINUE
  395 IF(ITRFLG.EQ.-100) GO TO 930
C***********END OF PRINTING INPUT PARAMETERS**********************************
C***********BEGIN CALCULATION OF NEW VELOCITIES*******************************
  400 IF(MDFLG.EQ.-999) GO TO 992
      AVERO=10000.
      DELERO=10000.
      ITRPRN=0
      ITRNEW=0
      IFLGPT=0
      ISFLG=0
      IF(ISGRA.EQ.1) ISFLG = -99
      DO 420 IPIXEL=1,NPXTOT
      LGLCOR(IPIXEL)=.TRUE.
      WTSMF(IPIXEL)=0.0
  420 SMFPIX(IPIXEL)=0.0
C*******FIX VELOCITY IN DESIGNATED NUMBER OF COLUMNS*****
      DO 422 IFXLFT=1,NFXLFT
      DO 422 IPIXV=1,NPIXV
      IPIXEL = IFXLFT + (IPIXV-1)*NPIXH
  422 LGLCOR(IPIXEL) = .FALSE.
      DO 424 IFXRHT=1,NFXRHT
      DO 424 IPIXV=1,NPIXV
      IPIXEL = NPIXH-(IFXRHT-1) + (IPIXV-1)*NPIXH
  424 LGLCOR(IPIXEL) = .FALSE.
C********************************************************
  430 IARSG=0
      NTOTSG=0
      DO 435 I=1,NPXTOT
  435 COR(I)=0.
C*******START MAIN DO LOOP FOR CALCULATION OF PATH SEGMENTS*****
      DO 640 IEXPTT=1,NEXPTT
      CAL = 0.0
       IF(MSTOR.EQ.1 .AND. ITRNEW.GT.0) GO TO 610
       IF(MDATA.NE.1) THEN
       IPST=IDEXTTT(IEXPTT)
       IF(IDPST(IPST).EQ.IDEXTTT(IEXPTT)) GO TO 448
       DO 440 IPST=1,NPST
  440  IF(IDPST(IPST).EQ.IDEXTTT(IEXPTT)) GO TO 448
       WRITE(*,445) IDEXTTT(IEXPTT),IEXPTT
       WRITE(2,445) IDEXTTT(IEXPTT),IEXPTT
  445  FORMAT(' DID NOT FIND TRANSMITTER POSITION NUMBER',I4,
     1 ' FOR TRAVEL TIME',I6,/,' ERROR IS OFTEN FROM TOO SMALL A DIMENSI
     2ON, SO AN ARRAY IS WRITTEN OVER.')
       GO TO 999
  448  IPSR=IDEXTTR(IEXPTT)
       IF(IDPSR(IPSR).EQ.IDEXTTR(IEXPTT)) GO TO 460
  449  DO 450 IPSR=1,NPSR
  450  IF(IDPSR(IPSR).EQ.IDEXTTR(IEXPTT)) GO TO 460
       WRITE(*,455) IDEXTTR(IEXPTT),IEXPTT
       WRITE(2,455) IDEXTTR(IEXPTT),IEXPTT
  455  FORMAT(' DID NOT FIND RECEIVER POSITION NUMBER',I4,
     1 ' FOR TRAVEL TIME',I6,/,' ERROR IS OFTEN FROM TOO SMALL A DIMENSI
     2ON, SO AN ARRAY IS WRITTEN OVER.')
       GO TO 999
  460  POSTXA=POSTX(IPST)
       POSTYA=POSTY(IPST)
       POSTZA=POSTZ(IPST)
       POSRXA=POSRX(IPSR)
       POSRYA=POSRY(IPSR)
       POSRZA=POSRZ(IPSR)
       ELSE
       POSTXA=POSTX(IEXPTT)
       POSTYA=POSTY(IEXPTT)
       POSTZA=POSTZ(IEXPTT)
       POSRXA=POSRX(IEXPTT)
       POSRYA=POSRY(IEXPTT)
       POSRZA=POSRZ(IEXPTT)
       END IF
        IF(ADJDIS.NE.0.) THEN
        DFACX=POSRXA-POSTXA
        DFACY=POSRYA-POSTYA
        DPTHH=SQRT(DFACX*DFACX + DFACY*DFACY)
        POSRXA=POSRXA+DFACX*ADJDIS/DPTHH
        POSRYA=POSRYA+DFACY*ADJDIS/DPTHH
        END IF
       DFACX=POSTXA-POSP1X
       DFACY=POSTYA-POSP1Y
       HYP1S=DFACX*DFACX + DFACY*DFACY
       DFACX=POSTXA-POSPNX
       DFACY=POSTYA-POSPNY
       HYPNS=DFACX*DFACX + DFACY*DFACY
       POSTH=.5*(HYP1S-HYPNS+DPIXHS)/DPIXH
       POSTV=POSTZA-POSP1Z
       DFACX=POSRXA-POSP1X
       DFACY=POSRYA-POSP1Y
       HYP1S=DFACX*DFACX + DFACY*DFACY
       DFACX=POSRXA-POSPNX
       DFACY=POSRYA-POSPNY
       HYPNS=DFACX*DFACX + DFACY*DFACY
       OFFNH=.5*(HYPNS-HYP1S+DPIXHS)/DPIXH
       POSRH=DPIXH-OFFNH
       POSRV=POSRZA-POSP1Z
C*NOV 87*IF RECEIVER IS LEFT OF TRANSMITTER, EXCHANGE POSITIONS.*****
      IFLGEX = 0
      IF(POSRH.LT.POSTH) THEN
       IFLGEX = 1
       TEMPH = POSRH
       TEMPV = POSRV
       POSRH = POSTH
       POSRV = POSTV
       POSTH = TEMPH
       POSTV = TEMPV
      END IF
C**********************************************************************
      IROWT=NINT(POSTV/EPIXV)+1
      IROWR=NINT(POSRV/EPIXV)+1
      ICOLT=NINT((POSTH+.00001)/EPIXH)+1
      ICOLR=NINT((POSRH-.00001)/EPIXH)+1
       IF(IROWT.LT.1 .OR. IROWR.LT.1) THEN
       WRITE(*,466) IEXPTT
       WRITE(2,466) IEXPTT
  466  FORMAT(/,' IROWT OR IROWR IS LESS THAN 1 FOR TRAVEL TIME NUMBER '
     1 ,I5,/,' MOVE PIXEL 1 (TOP LEFT) UPWARD.  EXECUTION TERMINATED.')
       END IF
       IF(IROWT.GT.NPIXV .OR. IROWR.GT.NPIXV) THEN
       WRITE(*,468) IEXPTT
       WRITE(2,468) IEXPTT
  468  FORMAT(/,' IROWT OR IROWR IS GREATER THAN NPIXV FOR TRAVEL TIME N
     1UMBER ',I5,/,' MOVE THE LAST PIXEL (BOTTOM RIGHT) DOWNWARD.  EXECU
     1TION TERMINATED.')
       END IF
       IF(ICOLT.LT.1) THEN
       WRITE(*,470) ICOLT,IEXPTT
       WRITE(2,470) ICOLT,IEXPTT
  470  FORMAT(/,' ICOLT = ',I6,' FOR TRAVEL TIME NUMBER ',
     1 I5,/,' MOVE PIXEL 1 (TOP LEFT) TO LEFT.  EXECUTION TERMINATED.')
       GO TO 999
       END IF
       IF(ICOLR.GT.NPIXH) THEN
       WRITE(*,472) ICOLR,IEXPTT
       WRITE(2,472) ICOLR,IEXPTT
  472  FORMAT(/,' ICOLR = ',I6,' ,WHICH IS GREATER THAN NPIXH, FOR TRAVE
     1L TIME NUMBER ',I5,/,' MOVE THE LAST PIXEL (BOTTOM RIGHT) TO RIGHT
     2.  EXECUTION TERMINATED.')
       GO TO 999
       END IF
      NSCROS=IROWR-IROWT
      NACROS=IABS(NSCROS)
      IA=0
      IF(NSCROS.GT.0)IA=1
      IF(NSCROS.LT.0)IA=-1

      DFACH=POSRH-POSTH
      DFACV=POSRV-POSTV
      DPATH=SQRT(DFACH*DFACH + DFACV*DFACV)
      DFACX=POSRXA-POSTXA
      DFACY=POSRYA-POSTYA
      DFACZ=POSRZA-POSTZA
      DPTHTL=SQRT(DFACX*DFACX + DFACY*DFACY + DFACZ*DFACZ)
      EXTADJ(IEXPTT)= (EXPTT(IEXPTT)+ADJTT)*DPATH/DPTHTL
      IF(MWT.NE.1) WTFAC=1./DPATH
      IF(MWT.EQ.1) WTFAC=WEIGHT(IEXPTT)/DPATH
C*NOV 87*IF TRANS AND REC ARE IN SAME COLUMN, USE SPECIAL ALGORITHM.***
      IF (ICOLT.EQ.ICOLR) THEN
C***JUNE 90**IF TRANS AND REC ARE ALSO IN SAME COLUMN, ONLY ONE SEG D.
        IF(IROWT.EQ.IROWR) THEN
      D(1)=SQRT((POSRV-POSTV)*(POSRV-POSTV)+(POSRH-POSTH)*(POSRH-POSTH))
        IPIXEL = ICOLT + (IROWT-1)*NPIXH
        CAL = D(1)/V(IPIXEL)
        IPIXSG(1) = IPIXEL
        NSG = 1
        GO TO 575
        END IF
C***END OF JUNE 90 REVISION
       SECV = SQRT((POSRV-POSTV)*(POSRV-POSTV) + (POSRH-POSTH)*
     1  (POSRH-POSTH))/(POSRV-POSTV)
       ABSECV = ABS(SECV)
       VERDT = .5*EPIXV - POSTV*IA + (IROWT-1)*EPIXV*IA
       D(1) = VERDT*ABSECV
       IPIXEL = ICOLT + (IROWT-1)*NPIXH
       CAL = CAL + D(1)/V(IPIXEL)
       IF (MBORE.EQ.1) LGLCOR(IPIXEL) = .FALSE.
       IPIXSG(1) = IPIXEL
        DO 478 JSG = 2, NACROS
        D(JSG) = EPIXV*ABSECV
        IPIXEL = ICOLT + (IROWT-1)*NPIXH + IA*(JSG-1)*NPIXH
        CAL = CAL + D(JSG)/V(IPIXEL)
  478   IPIXSG(JSG) = IPIXEL
       NSG = NACROS + 1
       VERDR = .5*EPIXV + POSRV*IA - (IROWR-1)*EPIXV*IA
       D(NSG) = VERDR*ABSECV
       IPIXEL = ICOLR + (IROWR-1)*NPIXH
       CAL = CAL + D(NSG)/V(IPIXEL)
       IF(MBORE.EQ.1) LGLCOR(IPIXEL) = .FALSE.
       IPIXSG(NSG) = IPIXEL
       GO TO 575
      END IF
C*END OF NOV 87 REVISION **********************************************
      DSEC=DPATH/DFACH
  480 SLOPE=ABS(DFACV/DFACH)
      DO 488 ICROS=1,NACROS
      TCROS(ICROS)=(.5*EPIXV-POSTV*IA+
     1(IROWT-1)*EPIXV*IA+(ICROS-1)*EPIXV)/SLOPE
      E1CROS=TCROS(ICROS)-(ICOLT-.5)*EPIXH+POSTH
      IF(E1CROS.GT.0)GO TO 485
      ICOL(ICROS)=ICOLT
      IF(ICROS.EQ.1) EDCROS(ICROS)=TCROS(ICROS)-POSTH+(ICOLT-1)*EPIXH
      IF(ICROS.GT.1)EDCROS(ICROS)=TCROS(ICROS)-TCROS(ICROS-1)
      GO TO 488
  485 ICOL(ICROS)=INT(E1CROS/EPIXH)+1+ICOLT
      EDCROS(ICROS)=E1CROS-(ICOL(ICROS)-1-ICOLT)*EPIXH
  488  CONTINUE
  490 ICOL(NACROS+1)=ICOLR
C
C*****START CALCULATIONS FOR COLUMN ICOLT*****
  500  ISG=1
       JPIXH=ICOLT
       ICROS=1
      IF (ICOL(ICROS).EQ.JPIXH) THEN
        D(1)=TCROS(ICROS)*DSEC
  505   IF(ICROS.EQ.NACROS) GO TO 510
          IF(ICOL(ICROS+1).NE.ICOL(ICROS)) GO TO 510
            ISG=ISG+1
            D(ISG)=(TCROS(ICROS+1)-TCROS(ICROS))*DSEC
            ICROS=ICROS+1
          GO TO 505
  510   ISG=ISG+1
        D(ISG)=((ICOLT-.5)*EPIXH-POSTH-TCROS(ICROS))*DSEC
        ICROS=ICROS+1
      ELSE
        D(1)=((ICOLT-.5)*EPIXH-POSTH)*DSEC
      END IF
      ISGT=ISG
      DO 520 JSG=1,ISGT
      IPIXEL=ICOLT+(IROWT-1)*NPIXH+IA*(JSG-1)*NPIXH
      CAL=CAL+D(JSG)/V(IPIXEL)
      IF(MBORE.EQ.1) LGLCOR(IPIXEL)=.FALSE.
  520 IPIXSG(JSG)=IPIXEL
C*****END OF CALCULATIONS FOR COLUMN ICOLT*****
C*****START CALCULATIONS FOR COLUMN ICOLT+1 THROUGH ICOLR-1*****
      JPIXH=ICOLT+1
C COLUMNS WHERE PATH DOES NOT CROSS PIXEL ROW BOUNDARY.
  525   DO 530 JJPIXH=JPIXH,ICOL(ICROS)-1
          ISG=ISG+1
          D(ISG)=EPIXH*DSEC
          IPIXEL=JJPIXH+(IROWT-1)*NPIXH+(ICROS-1)*IA*NPIXH
          CAL=CAL+D(ISG)/V(IPIXEL)
  530     IPIXSG(ISG)=IPIXEL
        JPIXH=ICOL(ICROS)
        IF(JPIXH.EQ.ICOLR) GO TO 550
C  PIXEL ABOVE POINT WHERE PATH CROSSES PIXEL ROW BOUNDARY.
  535 ISG=ISG+1
      D(ISG)=EDCROS(ICROS)*DSEC
      IPIXEL=ICOL(ICROS)+(IROWT-1)*NPIXH+(ICROS-1)*IA*NPIXH
      CAL=CAL+D(ISG)/V(IPIXEL)
      IPIXSG(ISG)=IPIXEL
      IF (ICROS.EQ.NACROS) GO TO 545
  540   IF (ICOL(ICROS+1).NE.ICOL(ICROS)) GO TO 545
          ISG=ISG+1
          D(ISG)=(EDCROS(ICROS+1)-EDCROS(ICROS))*DSEC
          IPIXEL=ICOL(ICROS)+(IROWT-1)*NPIXH+ICROS*IA*NPIXH
          CAL=CAL+D(ISG)/V(IPIXEL)
          IPIXSG(ISG)=IPIXEL
          ICROS=ICROS+1
        GO TO 540
C  PIXEL BELOW POINT WHERE PATH CROSSES PIXEL ROW BOUNDARY.
  545 ISG=ISG+1
      D(ISG)=(EPIXH-EDCROS(ICROS))*DSEC
      IPIXEL=ICOL(ICROS)+(IROWT-1)*NPIXH+ICROS*IA*NPIXH
      CAL=CAL+D(ISG)/V(IPIXEL)
      IPIXSG(ISG)=IPIXEL
        JPIXH=ICOL(ICROS)+1
        ICROS=ICROS+1
        GO TO 525

C*****START CALCULATIONS FOR COLUMN ICOLR*****
  550 ISGR=ISG+1
      IF(ICROS.GT.NACROS) GO TO 565
        ISG=ISG+1
        D(ISG)=EDCROS(ICROS)*DSEC
  555     IF (ICROS.GE.NACROS) GO TO 560
           ISG=ISG+1
           D(ISG)=(EDCROS(ICROS+1)-EDCROS(ICROS))*DSEC
           ICROS=ICROS+1
          GO TO 555
  560   ISG=ISG+1
       D(ISG)=(POSRH-DPIXH+(NPIXH-ICOLR+.5)*EPIXH-EDCROS(NACROS))
     1 *DSEC
       GO TO 567
  565   ISG=ISG+1
        D(ISG)=(POSRH-DPIXH+(NPIXH-ICOLR+.5)*EPIXH)*DSEC

  567 NSG=ISG
      DO 570 JSG=ISGR,NSG
      IPIXEL=ICOLR+(IROWR-1)*NPIXH-IA*(ISG-JSG)*NPIXH
      CAL=CAL+D(JSG)/V(IPIXEL)
      IF(MBORE.EQ.1) LGLCOR(IPIXEL)=.FALSE.
  570 IPIXSG(JSG)=IPIXEL
C*****END CALCULATIONS FOR COLUMN ICOLR*****
  575 NTOTSG=NTOTSG+NSG
C*****PRINT PATH SEGMENT LENGTHS IF IPSEG = 1 ******
      CONTINUE
       IF (IPSEG.EQ.1 .AND. ITRNEW.EQ.0) THEN
        IFLGPT=1
        IF(MDATA.NE.1)THEN
        IF (IFLGEX.EQ.0) WRITE(2,580)
        IF (IFLGEX.EQ.1) WRITE(2,581)
  580   FORMAT(' TRANS IDENT, ROW, COL    REC IDENT, ROW, COL   PIXEL
     1   PATH SEG LENGTH')
  581   FORMAT('   REC IDENT, ROW, COL  TRANS IDENT, ROW, COL   PIXEL
     1   PATH SEG LENGTH')
         DO 583 JD=1,NSG
         JJ=IPIXSG(JD)
  583   WRITE(2,585) IDEXTTT(IEXPTT),IROWT,ICOLT,IDEXTTR(IEXPTT),IROWR,
     1 ICOLR,JJ,D(JD)
  585   FORMAT(1X,I9,I7,I5,I11,I7,I5,I7,7X,G11.5)
         ELSE
         WRITE(2,590)
  590    FORMAT(' PATH NUM  TRANS ROW,COL  REC ROW,COL   PIXEL  PATH SEG
     1 LENGTH')
         DO 593 JD=1,NSG
         JJ=IPIXSG(JD)
  593    WRITE(2,595) IDPATH(IEXPTT),IROWT,ICOLT,IROWR,ICOLR,JJ,D(JD)
  595 FORMAT(1X,I5,I11,I5,I11,I5,I5,6X,G11.5)
        END IF
       END IF
C****************************************
       IF(ITRMAX.EQ.-1) THEN
       EXPTT(IEXPTT)=CAL
       GO TO 636
       END IF
C*****CALCULATE SUMS OF CORRECTION FACTORS FOR EACH PIXEL*******
      IF(MSTOR.NE.1 .OR. ITRNEW.NE.0) GO TO 627
      DO 605 JD=1,NSG
      IARSG=IARSG+1
      DP(IARSG)=D(JD)
  605 IDPIX(IARSG)=IPIXSG(JD)
      IPP(IEXPTT)=NSG
      DPPATH(IEXPTT)=DPATH
      GO TO 627
  610 CONTINUE
      IARSGS=IARSG
      DO 615 JD=1,IPP(IEXPTT)
      IARSG=IARSG+1
      IDP=IDPIX(IARSG)
  615 CAL=CAL+DP(IARSG)/V(IDP)
      DELT(IEXPTT)=EXTADJ(IEXPTT)-CAL
      RATIO(IEXPTT)=EXTADJ(IEXPTT)/CAL
      IARSG=IARSGS
      DO 620 JD=1,IPP(IEXPTT)
      IARSG=IARSG+1
      IF(MWT.NE.1) WTFAC=1./DPPATH(IEXPTT)
      IF(MWT.EQ.1 )WTFAC=WEIGHT(IEXPTT)/DPPATH(IEXPTT)
      IDP=IDPIX(IARSG)
      IF(MCOR.NE.1) COR(IDP)=COR(IDP)+DP(IARSG)*RATIO(IEXPTT)*
     1 WTFAC
  620 IF(MCOR.EQ.1) COR(IDP)=COR(IDP)+DP(IARSG)*DELT(IEXPTT)*
     1 WTFAC/DPPATH(IEXPTT)
      GO TO 640
  627 DELT(IEXPTT)=EXTADJ(IEXPTT)-CAL
      RATIO(IEXPTT)=EXTADJ(IEXPTT)/CAL
      DO 630 JV=1,NSG
      JJ=IPIXSG(JV)
      IF(MCOR.NE.1) COR(JJ)=COR(JJ)+D(JV)*RATIO(IEXPTT)*WTFAC
      IF(MCOR.EQ.1) COR(JJ)=COR(JJ)+D(JV)*DELT(IEXPTT)*WTFAC/
     1 DPATH
      IF(ISG.GT.IPMAX) IPMAX=NSG
  630 CONTINUE
  635 IF(ITRNEW.GT.0) GO TO 640
  636 IF(NSG.GT.IPMAX) IPMAX=NSG
      DO 637 JV=1,NSG
      JJ=IPIXSG(JV)
      IF(MWT.EQ.1) WTSMF(JJ)=WTSMF(JJ)+D(JV)*WEIGHT(IEXPTT)/DPATH
  637 SMFPIX(JJ)=SMFPIX(JJ)+D(JV)/DPATH
  640 CONTINUE
C*****END OF CALCULATION OF SUMS OF CORRECTION FACTORS FOR PIXELS*************
      IF(ITRMAX.EQ.-1) THEN
      IFLGPT = 1
      GO TO 800
      END IF
C***********CHECK FOR SUFFICIENT CONVERGENCE.**********************************
  650 IFLGPT = 0
      AVERN=0.
      DO 660 IJ=1,NEXPTT
  660 AVERN=AVERN+ABS(DELT(IJ))
      AVERN=AVERN/NEXPTT
      IF (AVERN.LT.TOLABS .OR. ITRSAV.GE.ITRMAX) IFLGPT = 1
      DELERN=AVERO-AVERN
      IF(DELERN.LT.TOLINC .AND. DELERO.LT.TOLINC) IFLGPT = 1
      AVERO=AVERN
      DELERO=DELERN
      IF(IFLGPT.EQ.1 .OR. ITRPRN.EQ.10) GO TO 800
C
C*****MAKE CORRECTIONS TO VELOCITIES*****
  680 DO 690 IPIXEL=1,NPXTOT
      IF(.NOT.LGLCOR(IPIXEL)) GO TO 690
      IF(SMFPIX(IPIXEL).LT..00001) GO TO 690
      IF(MWT.NE.1) COR(IPIXEL)=COR(IPIXEL)*RELAX/SMFPIX(IPIXEL)
      IF(MWT.EQ.1) COR(IPIXEL)=COR(IPIXEL)*RELAX/WTSMF(IPIXEL)
      IF(MCOR.NE.1) V(IPIXEL)=V(IPIXEL)/COR(IPIXEL)
      IF(MCOR.EQ.1) V(IPIXEL)=V(IPIXEL)/(1.+ V(IPIXEL)*COR(IPIXEL))
  690 CONTINUE
      ITRSAV=ITRSAV+1
      ITRPRN=ITRPRN+1
      ITRNEW=ITRNEW+1
C*****AVERAGE OVER DESIGNATED NUMBER OF ROWS AT TOP AND BOTTOM********
      DO 725 ICTOP=1,NLITOP
      VAVTOP=0.
      NPX=0
      DO 710 IPIXH=1,NPIXH
      IPIXEL = IPIXH + (ICTOP-1)*NPIXH
      IF(SMFPIX(IPIXEL).LT..00001) GO TO 710
      NPX=NPX+1
      VAVTOP=VAVTOP+V(IPIXEL)
  710 CONTINUE
       IF(NPX.GT.0) THEN
       VAVTOP=VAVTOP/NPX
       DO 720 IPIXH=1,NPIXH
       IPIXEL = IPIXH + (ICTOP-1)*NPIXH
  720  IF(LGLCOR(IPIXEL)) V(IPIXEL)=VAVTOP
       END IF
  725 CONTINUE
      DO 745 ICBOT=1,NLIBOT
      VAVBOT=0.
      NPX=0
      DO 735 IPIXH=1,NPIXH
      IPIXEL = IPIXH + (NPIXV-ICBOT)*NPIXH
      IF(SMFPIX(IPIXEL).LT..00001) GO TO 735
      NPX=NPX+1
      VAVBOT=VAVBOT+V(IPIXEL)
  735 CONTINUE
       IF(NPX.GT.0) THEN
       VAVBOT=VAVBOT/NPX
       DO 740 IPIXH=1,NPIXH
       IPIXEL = IPIXH + (NPIXV-ICBOT)*NPIXH
  740  IF(LGLCOR(IPIXEL)) V(IPIXEL)=VAVBOT
       END IF
  745 CONTINUE

      IF(MSMTH.NE.1) GO TO 776
C*********SMOOTHING*****************
  750 DO 770 IPIXH = 1, NPIXH
      DO 770 IPIXV = 1, NPIXV
      IPIXEL = IPIXH + (IPIXV-1)*NPIXH
      IDHMIN = -1
      IDHMAX = 1
      IDVMIN = -1
      IDVMAX = 1
      IF(IPIXH.EQ.1) IDHMIN = 0
      IF(IPIXH.EQ.NPIXH) IDHMAX = 0
      IF(IPIXV.EQ.1) IDVMIN = 0
      IF(IPIXV.EQ.NPIXV) IDVMAX = 0
      VSMTH(IPIXEL) = 0.
      NPXSM = 0
      DO 765 IDH = IDHMIN,IDHMAX
      DO 765 IDV = IDVMIN,IDVMAX
      IPXSM = IPIXEL + IDH + IDV*NPIXH
      MLWT=1
      IF(IDH.EQ.0 .OR. IDV.EQ.0) MLWT=4
      IF(IDH.EQ.0.AND.IDV.EQ.0) THEN
      MLWT=20
      GO TO 764
      END IF
      IF(SMFPIX(IPXSM).LT..00001) GO TO 765
  764 VSMTH(IPIXEL)=VSMTH(IPIXEL) + MLWT*V(IPXSM)
      NPXSM = NPXSM+MLWT
  765 CONTINUE
  770 VSMTH(IPIXEL) = VSMTH(IPIXEL)/NPXSM
      DO 775 IPIXEL = 1, NPXTOT
  775 IF(LGLCOR(IPIXEL)) V(IPIXEL) = VSMTH(IPIXEL)
C**********CONSTRAINING VELOCITY WITHIN LOWER AND UPPER LIMITS*********
  776 IF(VLOCAL.NE.-99.) THEN
      DO 777 IPIXEL = 1, NPXTOT
  777 IF(V(IPIXEL).LT.VLOCAL) V(IPIXEL)=VLOCAL
      END IF
      IF(VHICAL.NE.-99.) THEN
      DO 778 IPIXEL = 1,NPXTOT
  778 IF(V(IPIXEL).GT.VHICAL) V(IPIXEL)=VHICAL
      END IF
  780 IF(IFLGPT.NE.1) GO TO 430

C********SAVE ITERATION RESULTS IN DATA FILE**********************************
  800  OPEN(4,FILE=DATFIL,ERR=128,IOSTAT=IOS)
       IF(ITRMAX.EQ.-1) THEN
       ITRFLG=-1
       ITRMAX=5
       ITRSAV=0
       END IF
  805  WRITE(4,806) HEADER
  806  FORMAT(1X,A79)
       WRITE(4,808) MDATA,MWT,MSTOR,MCOR,MBORE,MSMTH,MDIF
  808  FORMAT(8I10)
       WRITE(4,810) ITRMAX,ITRSAV,TOLABS,TOLINC,RELAX
  810  FORMAT(I10,I10,1X,F11.7,1X,F11.7,1X,F11.6)
       WRITE(4,812) IPDAT,IPIVG,IPFRC,IPERR,IPSEG,IPGRA,ISGRA
  812  FORMAT(7I10)
       WRITE(4,814) POSP1X,POSP1Y,POSP1Z,POSPNX,POSPNY,POSPNZ
  814  FORMAT(6(1X,F11.4))
       WRITE(4,816) NPIXH,NPIXV,NLITOP,NLIBOT,NFXLFT,NFXRHT
  816  FORMAT(6I10)
       IF(MDATA.NE.1) THEN
       WRITE(4,830) NPST,NPSR,ADJDIS
  830  FORMAT(2I10,F11.5)
       DO 832 IPST=1,NPST
  832  WRITE(4,834) IDPST(IPST),POSTX(IPST),POSTY(IPST),POSTZ(IPST)
  834  FORMAT(I10,3(1X,F11.4))
       DO 836 IPSR=1,NPSR
  836  WRITE(4,834) IDPSR(IPSR),POSRX(IPSR),POSRY(IPSR),POSRZ(IPSR)
       WRITE(4,854) NEXPTT,ADJTT
       IF(MWT.NE.1) THEN
       DO 838 IEXPTT=1,NEXPTT
  838  WRITE(4,852) IDEXTTT(IEXPTT),IDEXTTR(IEXPTT),EXPTT(IEXPTT)
         ELSE
       DO 850 IEXPTT=1,NEXPTT
  850  WRITE(4,852) IDEXTTT(IEXPTT),IDEXTTR(IEXPTT),EXPTT(IEXPTT),
     1  WEIGHT(IEXPTT)
  852  FORMAT(2I10,F15.6,F5.2)
        END IF
       ELSE
       WRITE(4,854) NEXPTT,ADJDIS,ADJTT
  854  FORMAT(I10,2F11.5)
        IF(MWT.NE.1) THEN
       DO 860 IEXPTT=1,NEXPTT
  860  WRITE(4,862)IDPATH(IEXPTT),POSTX(IEXPTT),POSTY(IEXPTT),
     1 POSTZ(IEXPTT),POSRX(IEXPTT),POSRY(IEXPTT),POSRZ(IEXPTT),
     1 EXPTT(IEXPTT)
  862 FORMAT(I8,6(1X,F9.3),F11.6)
        ELSE
       DO 864 IEXPTT=1,NEXPTT
  864  WRITE(4,866)IDPATH(IEXPTT),POSTX(IEXPTT),POSTY(IEXPTT),
     1 POSTZ(IEXPTT),POSRX(IEXPTT),POSRY(IEXPTT),POSRZ(IEXPTT),
     1 EXPTT(IEXPTT),WEIGHT(IEXPTT)
  866 FORMAT(I8,6(1X,F9.3),F11.6,F5.2)
       END IF
      END IF
      WRITE(4,880)SMFPLL,VLOCAL,VHICAL,VLODSP,VHIDSP,VLODIF,VHIDIF
  880 FORMAT(1X,7F10.3,'   3   ')
       DO 882 IPIXV=1,NPIXV
  882  WRITE(4,884)IPIXV,(V(IPIXH+(IPIXV-1)*NPIXH),IPIXH=1,NPIXH)
  884 FORMAT(I5,80F10.6)
       CLOSE(4)
      IF(ICRFLG.EQ.-99) GO TO 120
      IF(ISFLG.EQ.-99) GO TO 957
  885 IF(ITRFLG.EQ.-1) THEN
       WRITE(*,886) DATFIL
  886  FORMAT(' SYNTHETIC DATA SAVED IN ',A12)
       GO TO 900
      END IF
      WRITE(*,888) OUTFIL,ITRSAV,AVERN,DELERN
  888 FORMAT(1X,A12,' ITER',I5,'  AVE ERROR=',G10.3,'  CHANGE/ITER IN AV
     1E ERR=',G9.2)
       ITRPRN=0
  890  IF(IFLGPT.NE.1) GO TO 680
C
C***********PRINT RESULTS*****************************************************
  900 ISFLG = 0
      IF(IPFRC.EQ.1) THEN
      WRITE(2,902)
  902 FORMAT(/,' SUM OF FRACTIONS OF PATH SEGMENTS IN EACH PIXEL')
      DO 904 IPIXV=1,NPIXV
  904 WRITE(2,906) (SMFPIX(IPIXH+(IPIXV-1)*NPIXH),IPIXH=1,NPIXH)
  906 FORMAT(1X,80F5.2)
      END IF
      IF(IPERR.EQ.1) THEN
      WRITE(2,910)
  910 FORMAT(//,' ADJUSTED EXPERIMENTAL MINUS CALCULATED TRAVEL TIMES')
      IF(MDATA.NE.1) THEN
      WRITE(2,*)(' TRANS IDENT  REC IDENT    EXPERIMENTAL     EXP-CALC')
      DO 912 IEXPTT=1,NEXPTT
  912 WRITE(2,914) IDEXTTT(IEXPTT),IDEXTTR(IEXPTT),EXTADJ(IEXPTT),
     1 DELT(IEXPTT)
  914 FORMAT(I11,I11,F18.5,F13.5)
      ELSE
      WRITE(2,*)(' PATH IDENT    EXPERIMENTAL     EXP-CALC')
      DO 916 IEXPTT=1,NEXPTT
  916 WRITE(2,918) IDPATH(IEXPTT),EXTADJ(IEXPTT),DELT(IEXPTT)
  918 FORMAT(I10,F18.5,F13.5)
      END IF
      END IF

  920 IF(ITRFLG.EQ.-1) THEN
       ITRFLG=-100
       GO TO 328
       END IF
  930 WRITE(2,932) IPMAX
  932 FORMAT(//,' DIMENSION NEEDED FOR D(ISG) IS ISG =',I4)
      IF(IARSG.EQ.0) IARSG=NTOTSG
      WRITE(2,934) IARSG,NEXPTT
  934 FORMAT(' IF MSTOR = 1, DIMENSION NEEDED FOR DP(IARSG), IDPIX(IARSG
     1),',/,' IPP(NEXPTT), AND DPPATH(NEXPTT) IS IARSG =',I6,
     2'   NEXPTT =',I5)
      IF(ITRFLG.EQ.-100) THEN
      WRITE(2,935)
  935 FORMAT(/,' VELOCITIES IN EACH PIXEL')
      GO TO 942
      END IF
  936 RMS=0.
      DO 937 IEXPTT=1,NEXPTT
  937 RMS=RMS+DELT(IEXPTT)*DELT(IEXPTT)
      RMS=SQRT(RMS/NEXPTT)
      IF(ITRNEW.EQ.1)  WRITE(2,938) ITRSAV,AVERN
      IF(ITRNEW.GT.1)  WRITE(2,939) ITRSAV,AVERN,DELERN
  938 FORMAT(/,' NUMBER OF ITERATIONS= ',I5,17X,'   AVE ERROR=',G10.3)
  939 FORMAT(/,' NUMBER OF ITERATIONS= ',I5,17X,'   AVE ERROR=',G10.3,
     1 /,17X,       '                CHANGE/ITER IN AVE ERROR=',G10.3)
      WRITE(2,940) RMS
  940 FORMAT(' SQUARE ROOT OF (SUM OF SQUARES OF ERROR/NUMBER OF PATHS)=
     1',G10.3)
      WRITE(2,941)
  941 FORMAT(//,'  CALCULATED VELOCITIES FOR EACH PIXEL')
  942 WRITE(2,944) SMFPLL
  944 FORMAT(' ### INDICATES SUM OF PATH FRACTIONS LESS THAN OR EQUAL ',
     1 G9.3)
  946 DO 956 IPIXV=1,NPIXV
      DO 948 IPIXH=1,NPIXH
      IPIXEL=IPIXH+(IPIXV-1)*NPIXH
       IF(SMFPIX(IPIXEL).GT.SMFPLL) THEN
       WRITE(VALPHA(IPIXH),950) V(IPIXEL)
       ELSE
       WRITE(VALPHA(IPIXH),952)
       END IF
  948 CONTINUE
      WRITE(2,954) (VALPHA(IPIXH),IPIXH=1,NPIXH)
  950 FORMAT(F6.2)
  952 FORMAT('   ###')
  954 FORMAT(1X,80A6)
  956 CONTINUE

  957 IF(VLODSP.EQ.-99.) THEN
        VMIN=10000.
        DO 958 IPIXEL=1,NPXTOT
        IF(SMFPIX(IPIXEL).LE.SMFPLL) GO TO 958
        VMIN=AMIN1(VMIN,V(IPIXEL))
  958   CONTINUE
      ELSE
        VMIN=VLODSP
      END IF
      IF(VHIDSP.EQ.-99.) THEN
        VMAX=-10000.
        DO 960 IPIXEL=1,NPXTOT
        IF(SMFPIX(IPIXEL).LE.SMFPLL) GO TO 960
        VMAX=AMAX1(VMAX,V(IPIXEL))
  960   CONTINUE
      ELSE
        VMAX=VHIDSP
      END IF
      VDIFF=VMAX-VMIN
      DIV1=VMIN+0.2*VDIFF
      DIV2=VMIN+0.4*VDIFF
      DIV3=VMIN+0.6*VDIFF
      DIV4=VMIN+0.8*VDIFF
      IF(ISFLG.EQ.-99) GO TO 983
      IF(MDFLG.NE.-99) WRITE(2,962) VMIN,VMAX
  962 FORMAT(/,'  VELOCITIES SCALED TO RANGE 0 TO 9',/,
     1 ' VELOCITY CORRESPONDING TO 0 = ',G9.3,/,
     2 ' VELOCITY CORRESPONDING TO 9 = ',G9.3)
      IF(MDFLG.EQ.-99) WRITE(2,964) VMIN,VMAX
  964 FORMAT(/,'  VELOCITY DIFFERENCES SCALED TO RANGE 0 TO 9',/,
     1 ' VELOCITY DIFFERENCE CORRESPONDING TO 0 = ',G9.3,/
     2 ' VELOCITY DIFFERENCE CORRESPONDING TO 9 = ',G9.3)
      WRITE(2,966) SMFPLL
  966 FORMAT(' # INDICATES SUM OF PATH FRACTIONS LESS THAN OR EQUAL ',
     1 F7.4,/)
       IF(VDIFF.LT..00001) THEN
        DO 970 IPIXEL=1,NPXTOT
  970   IVSCAL(IPIXEL)=5
       ELSE
        SCALE=9./(VMAX-VMIN)
        DO 972 IPIXEL=1,NPXTOT
  972   IVSCAL(IPIXEL)=NINT((V(IPIXEL)-VMIN)*SCALE)
       END IF
      DO 980 IPIXV=1,NPIXV
      DO 974 IPIXH=1,NPIXH
      IPIXEL=IPIXH+(IPIXV-1)*NPIXH
       IF(SMFPIX(IPIXEL).GT.SMFPLL) THEN
       WRITE(VSCALE(IPIXH),975) IVSCAL(IPIXEL)
       ELSE
       WRITE(VSCALE(IPIXH),976)
       END IF
  974 CONTINUE
      IF(NPIXH.LT.40) WRITE(2,978) (VSCALE(IPIXH),IPIXH=1,NPIXH)
      IF(NPIXH.GE.40) WRITE(2,979) (VSCALE(IPIXH),IPIXH=1,NPIXH)
  975 FORMAT(I1)
  976 FORMAT('#')
  978 FORMAT(1X,39(1X,A1))
  979 FORMAT(1X,79A1)
  980 CONTINUE
      IF(IPGRA.EQ.1) THEN
      WRITE(2,981)
  981 FORMAT(//)
      IF(MDFLG.NE.-99) WRITE(2,*)('  GRAPH OF VELOCITIES')
      IF(MDFLG.EQ.-99) WRITE(2,*)('  GRAPH OF VELOCITY DIFFERENCES')
      WRITE(2,*)(' VELOCITY LIMITS FOR A PATTERN ARE ON EACH SIDE OF IT.
     1')
      END IF
      IF(IPGRA.EQ.1)WRITE(2,982) VMIN,CHAR(32),CHAR(32),DIV1,CHAR(176),
     1 CHAR(176),DIV2,CHAR(177),CHAR(177),DIV3,CHAR(178),CHAR(178),DIV4,
     2 CHAR(219),CHAR(219),VMAX
  982 FORMAT(/,'   BELOW <   ',F5.2,1X,2A1,F5.2,1X,2A1,F5.2,1X,2A1,
     1 F5.2,1X,2A1,F5.2,1X,2A1,F5.2,'   > ABOVE',/)
      IF(IPGRA.EQ.1) WRITE(2,966) SMFPLL
  983 DO 988 IPIXV=1,NPIXV
      DO 984 IPIXH=1,NPIXH
      IPIXEL=IPIXH+(IPIXV-1)*NPIXH
       IF(SMFPIX(IPIXEL).GT.SMFPLL) THEN
       WRITE(VSCALE(IPIXH),975) IVSCAL(IPIXEL)
        IF(V(IPIXEL).LT.VMIN) IASCI=60
        IF(V(IPIXEL).GE.VMIN .AND. V(IPIXEL).LT.DIV1) IASCI=32
        IF(V(IPIXEL).GE.DIV1 .AND. V(IPIXEL).LT.DIV2) IASCI=176
        IF(V(IPIXEL).GE.DIV2 .AND. V(IPIXEL).LE.DIV3) IASCI=177
        IF(V(IPIXEL).GT.DIV3 .AND. V(IPIXEL).LE.DIV4) IASCI=178
        IF(V(IPIXEL).GT.DIV4 .AND. V(IPIXEL).LE.VMAX) IASCI=219
        IF(V(IPIXEL).GT.VMAX) IASCI=62
         IF(IPIXV.EQ.1 .AND. IASCI.EQ.32) IASCI=196
         IF(IPIXV.EQ.NPIXV .AND. IASCI.EQ.32) IASCI=95
        WRITE(VSCALE(IPIXH),985) CHAR(IASCI)
       ELSE
       WRITE(VSCALE(IPIXH),976)
       END IF
  984 CONTINUE
      IF(IPGRA.EQ.1 .AND. ISFLG.NE.-99) THEN
      IF(NPIXH.LT.40) WRITE(2,987) (VSCALE(IPIXH),VSCALE(IPIXH),
     1 IPIXH=1,NPIXH)
      IF(NPIXH.GE.40) WRITE(2,987) (VSCALE(IPIXH),IPIXH=1,NPIXH)
      END IF
      IF(ISFLG.EQ.-99 .OR. ISFLG.EQ.-98) THEN
      IF(NPIXH.LT.40) WRITE(*,987) (VSCALE(IPIXH),VSCALE(IPIXH),
     1 IPIXH=1,NPIXH)
      IF(NPIXH.GE.40) WRITE(*,987) (VSCALE(IPIXH),IPIXH=1,NPIXH)
      END IF
  985 FORMAT(A1)
  986 FORMAT('CHAR(176)')
  987 FORMAT(1X,79A1)
  988 CONTINUE
      IF(ISFLG.EQ.-99 .OR. ISFLG.EQ.-98) WRITE(*,990) VMIN,CHAR(32),
     1 CHAR(32),DIV1,CHAR(176),CHAR(176),DIV2,CHAR(177),CHAR(177),
     2 DIV3,CHAR(178),CHAR(178),DIV4,CHAR(219),CHAR(219),VMAX
  990 FORMAT('  BELOW <    ',F5.2,1X,2A1,F5.2,1X,2A1,F5.2,1X,2A1,
     1 F5.2,1X,2A1,F5.2,1X,2A1,F5.2,'   > ABOVE')
      IF(ISFLG.EQ.-99) GO TO 885
C*****IF VELOCITIES WILL BE COMPARED, STORE VELOCITIES IN COR*****
      IF(MDIF.NE.1 .AND. MDFLG.EQ.0) GO TO 1940
      IF(MDFLG.EQ.-99) GO TO 996
      DO 991 IPIXEL=1,NPXTOT
  991 COR(IPIXEL)=V(IPIXEL)
      OPEN(3,FILE=OLDFIL,STATUS='OLD',ERR=128)
      MDFLG=-999
      GO TO 130
  992 DO 993 IPIXEL=1,NPXTOT
      V(IPIXEL)=COR(IPIXEL)-V(IPIXEL)
  993 CONTINUE
      WRITE(2,994) OLDFIL,SMFPLL
  994 FORMAT(//,'  VELOCITIES OF THIS RUN MINUS VELOCITIES FROM ',A12,
     1 /,' ### INDICATES SUM OF PATH FRACTIONS LESS THAN OR EQUAL ',
     2F7.4)
      VLODSP=VLODIF
      VHIDSP=VHIDIF
      MDFLG=-99
      ISFLG=-98
      GO TO 946
  996 RMS=0
      ABDIF=0
      NPX=0
      DO 997 IPIXEL=1,NPXTOT
      IF(SMFPIX(IPIXEL).LE.SMFPLL) GO TO 997
      ABDIF=ABDIF+ABS(V(IPIXEL))
      RMS=RMS+V(IPIXEL)*V(IPIXEL)
      NPX=NPX+1
  997 CONTINUE
      ABDIF=ABDIF/NPX
      RMS=SQRT(RMS/NPX)
      WRITE(2,998) SMFPLL,ABDIF,RMS
  998 FORMAT(/,'  FOR PIXELS WITH SUM OF FRACTIONS OF PATHS GREATER THAN
     1 ', G9.3,/,' THE AVERAGE ABSOLUTE VALUE OF VELOCITY DIFFERENCES =
     2',G9.3,/,' SQUARE ROOT (SUM OF DIFFERENCES SQUARED/NUMBER OF PIXEL
     3S) = ',G9.3)
C*JUNE 1990*ADDED OUTPUT FOR SURFER GRID FILE, GREATEST DEPTH IS ROW 1
 1940 WRITE(2,1942)
 1942 FORMAT(//,'  GRD FILE FOR THE CONTOURING PROGRAM SURFER.',/,
     1 ' ALL ### MUST BE CHANGED TO NUMBERS FOR SURFER.',/,
     2 ' ROW 1 REPRESENTS GREATEST DEPTH IN GRD FILE.',//,'DSAA')
      NX = NPIXH
      NY = NPIXV
      XLO = POSP1X
      XHI = POSPNX
      YLO = -POSPNZ
      YHI = -POSP1Z
      ZLO = VMIN
      ZHI = VMAX
      WRITE(2,1944) NX,NY,XLO,XHI,YLO,YHI,ZLO,ZHI
 1944 FORMAT(1X,2I10,/,1X,2F10.3,/,1X,2F10.3,/,1X,2F10.4)
 1946 DO 1956 IPIXV=NPIXV,1,-1
      DO 1948 IPIXH=1,NPIXH
      IPIXEL=IPIXH+(IPIXV-1)*NPIXH
       IF(SMFPIX(IPIXEL).GT.SMFPLL) THEN
       WRITE(VALPHA(IPIXH),950) V(IPIXEL)
       ELSE
       WRITE(VALPHA(IPIXH),952)
       END IF
 1948 CONTINUE
      WRITE(2,954) (VALPHA(IPIXH),IPIXH=1,NPIXH)
 1956 CONTINUE
C***END OF JUNE 1990 REVISION FOR SURFER GRD FILE***
  999 PRINT *,CHAR(7)
      PRINT *,CHAR(7)
      END
